from skimage import io, transform
from multiprocessing.dummy import Pool as ThreadPool

def rescale(root_new, root_old, img_path, ann_path, out_shape):
  try:
    img = io.imread(root_old+"/"+img_path)
  except Exception as E:
    print E
  h, w, _ = img.shape
  f_h, f_w = float(out_shape)/h, float(out_shape)/w
  trans_img = transform.rescale(img, (f_h, f_w))
  num_objs = 0
  with open(root_old+"/"+ann_path, 'r') as f:
    ann = f.readline()
    ann = ann.rstrip()
    ann = ann.split(' ')
    ann = [float(i) for i in ann]
    num_objs = len(ann) / 5
    for idx in xrange(num_objs):
      ann[idx * 5 + 0] = int(f_w * ann[idx * 5 + 0])
      ann[idx * 5 + 1] = int(f_h * ann[idx * 5 + 1])
      ann[idx * 5 + 2] = int(f_w * ann[idx * 5 + 2])
      ann[idx * 5 + 3] = int(f_h * ann[idx * 5 + 3])
    # Write the new annotations to file
    with open(root_new+"/"+ann_path, 'w') as f_new:
      for val in ann:
        f_new.write(str(val)+' ')
  # Save the new image
  io.imwrite(root_new+"/"+img_path, trans_img)

def preprocess():
  source = '/users2/Datasets/PASCAL_VOC/VOCdevkit/VOC2012_Resize/source.txt'
  root_old = '/users2/Datasets/PASCAL_VOC/VOCdevkit/VOC2012'
  root_new = '/users2/Datasets/PASCAL_VOC/VOCdevkit/VOC2012_Resize'
  out_shape = 416
  with open(source, 'r') as src:
    lines = src.readlines()
    print 'Processing {} images and annotations'.format(len(lines))
    for line in lines:
      line = line.rstrip()
      line = line.split(' ')
      img_path = line[0]
      ann_path = line[1]
      rescale(root_new, root_old, img_path, ann_path, out_shape)

if __name__ == '__main__':
  preprocess()



import numpy as np
import cv2 as cv

# add some external resources
cv.ovis.addResourceLocation("packs/Sinbad.zip")

# camera intrinsics
imsize = (800, 600)
K = np.diag([800, 800, 1])
K[:2, 2] = (400, 500) # offset pp

# observer scene
owin = cv.ovis.createWindow("VR", imsize)
cv.ovis.createGridMesh("ground", (10, 10), (10, 10))
owin.createEntity("ground", "ground", rot=(1.57, 0, 0))
owin.createCameraEntity("cam", K, imsize, 5)
owin.createEntity("sinbad", "Sinbad.mesh", tvec=(0, -5, 0), rot=(np.pi, 0, 0))  # externally defined mesh
owin.createLightEntity("sun", (0, 0, -100))

# setup and play idle animation
owin.setEntityProperty("sinbad", cv.ovis.ENTITY_ANIMBLEND_MODE, 1) # 1 = cumulative
owin.playEntityAnimation("sinbad", "IdleBase")
owin.playEntityAnimation("sinbad", "IdleTop")

# interaction scene
iwin = cv.ovis.createWindow("AR", imsize, cv.ovis.SCENE_SEPARATE | cv.ovis.SCENE_INTERACTIVE)
iwin.createEntity("sinbad", "Sinbad.mesh", tvec=(0, -5, 0), rot=(np.pi, 0, 0))
iwin.createLightEntity("sun", (0, 0, -100))
iwin.setCameraIntrinsics(K, imsize)

while cv.ovis.waitKey(1) != 27:
    R, t = iwin.getCameraPose()
    owin.setEntityPose("cam", t, R)

del iwin # must be destroyed in reverse creation order


#!/usr/bin/env python

# Python 2/3 compatibility
from __future__ import print_function

import os

import cv2 as cv

from tests_common import NewOpenCVTests

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
MODULE_DIR = os.path.join(SCRIPT_DIR, '../../../')

class shape_test(NewOpenCVTests):

    def test_computeDistance(self):

        a = cv.imread(os.path.join(MODULE_DIR, 'samples/data/shape_sample/1.png'), cv.IMREAD_GRAYSCALE)
        b = cv.imread(os.path.join(MODULE_DIR, 'samples/data/shape_sample/2.png'), cv.IMREAD_GRAYSCALE)
        if a is None or b is None:
            raise unittest.SkipTest("Missing files with test data")

        ca, _ = cv.findContours(a, cv.RETR_CCOMP, cv.CHAIN_APPROX_TC89_KCOS)
        cb, _ = cv.findContours(b, cv.RETR_CCOMP, cv.CHAIN_APPROX_TC89_KCOS)

        hd = cv.createHausdorffDistanceExtractor()
        sd = cv.createShapeContextDistanceExtractor()

        d1 = hd.computeDistance(ca[0], cb[0])
        d2 = sd.computeDistance(ca[0], cb[0])

        self.assertAlmostEqual(d1, 26.4196891785, 3, "HausdorffDistanceExtractor")
        self.assertAlmostEqual(d2, 0.25804194808, 3, "ShapeContextDistanceExtractor")

if __name__ == '__main__':
    NewOpenCVTests.bootstrap()



#!/usr/bin/env python
import os
import cv2 as cv
import numpy as np

from tests_common import NewOpenCVTests, unittest

class cudaobjdetect_test(NewOpenCVTests):
    def setUp(self):
        super(cudaobjdetect_test, self).setUp()
        if not cv.cuda.getCudaEnabledDeviceCount():
            self.skipTest("No CUDA-capable device is detected")

    @unittest.skipIf('OPENCV_TEST_DATA_PATH' not in os.environ,
                        "OPENCV_TEST_DATA_PATH is not defined")
    def test_hog(self):
        img_path = os.environ['OPENCV_TEST_DATA_PATH'] + '/gpu/caltech/image_00000009_0.png'
        npMat = cv.cvtColor(cv.imread(img_path),cv.COLOR_BGR2BGRA)

        cuMat = cv.cuda_GpuMat(npMat)
        cuHog = cv.cuda.HOG_create()
        cuHog.setSVMDetector(cuHog.getDefaultPeopleDetector())

        loc, conf = cuHog.detect(cuMat)
        self.assertTrue(len(loc) == len(conf) and len(loc) > 0 and len(loc[0]) == 2)

        loc = cuHog.detectWithoutConf(cuMat)
        self.assertTrue(len(loc) > 0 and len(loc[0]) == 2)

        loc = cuHog.detectMultiScaleWithoutConf(cuMat)
        self.assertTrue(len(loc) > 0 and len(loc[0]) == 4)

        cuHog.setGroupThreshold(0)
        loc, conf = cuHog.detectMultiScale(cuMat)
        self.assertTrue(len(loc) == len(conf) and len(loc) > 0 and len(loc[0]) == 4)

if __name__ == '__main__':
    NewOpenCVTests.bootstrap()


import os
import cv2 as cv
import numpy as np

from tests_common import NewOpenCVTests, unittest

class nvidiaopticalflow_test(NewOpenCVTests):
    def setUp(self):
        super(nvidiaopticalflow_test, self).setUp()
        if not cv.cuda.getCudaEnabledDeviceCount():
            self.skipTest("No CUDA-capable device is detected")

    @unittest.skipIf('OPENCV_TEST_DATA_PATH' not in os.environ,
                        "OPENCV_TEST_DATA_PATH is not defined")
    def test_calc(self):
        frame1 = os.environ['OPENCV_TEST_DATA_PATH'] + '/gpu/opticalflow/frame0.png'
        frame2 = os.environ['OPENCV_TEST_DATA_PATH'] + '/gpu/opticalflow/frame1.png'

        npMat1 = cv.cvtColor(cv.imread(frame1),cv.COLOR_BGR2GRAY)
        npMat2 = cv.cvtColor(cv.imread(frame2),cv.COLOR_BGR2GRAY)

        cuMat1 = cv.cuda_GpuMat(npMat1)
        cuMat2 = cv.cuda_GpuMat(npMat2)
        try:
            nvof = cv.cuda_NvidiaOpticalFlow_1_0.create(cuMat1.shape[1], cuMat1.shape[0], 5, False, False, False, 0)
            flow = nvof.calc(cuMat1, cuMat2, None)
            self.assertTrue(flow.shape[1] > 0 and flow.shape[0] > 0)
            flowUpSampled = nvof.upSampler(flow[0], cuMat1.shape[1], cuMat1.shape[0], nvof.getGridSize(), None)
            nvof.collectGarbage()
        except cv.error as e:
            if e.code == cv.Error.StsBadFunc or e.code == cv.Error.StsBadArg or e.code == cv.Error.StsNullPtr:
                self.skipTest("Algorithm is not supported in the current environment")
        self.assertTrue(flowUpSampled.shape[1] > 0 and flowUpSampled.shape[0] > 0)

if __name__ == '__main__':
    NewOpenCVTests.bootstrap()


import numpy as np
import cv2 as cv

# aruco
adict = cv.aruco.Dictionary_get(cv.aruco.DICT_4X4_50)
cv.imshow("marker", cv.aruco.drawMarker(adict, 0, 400))

# random calibration data. your mileage may vary.
imsize = (800, 600)
K = cv.getDefaultNewCameraMatrix(np.diag([800, 800, 1]), imsize, True)

# AR scene
cv.ovis.addResourceLocation("packs/Sinbad.zip") # shipped with Ogre

win = cv.ovis.createWindow("arucoAR", imsize, flags=0)
win.setCameraIntrinsics(K, imsize)
win.createEntity("figure", "Sinbad.mesh", (0, 0, 5), (1.57, 0, 0))
win.createLightEntity("sun", (0, 0, 100))

# video capture
cap = cv.VideoCapture(0)
cap.set(cv.CAP_PROP_FRAME_WIDTH, imsize[0])
cap.set(cv.CAP_PROP_FRAME_HEIGHT, imsize[1])

while cv.ovis.waitKey(1) != 27:
    img = cap.read()[1]
    win.setBackground(img)
    corners, ids = cv.aruco.detectMarkers(img, adict)[:2]

    cv.waitKey(1)

    if ids is None:
        continue

    rvecs, tvecs = cv.aruco.estimatePoseSingleMarkers(corners, 5, K, None)[:2]
    win.setCameraPose(tvecs[0].ravel(), rvecs[0].ravel(), invert=True)



import numpy as np
import cv2 as cv
import math

class ThParameters:
    def __init__(self):
        self.levelNoise=6
        self.angle=45
        self.scale10=5
        self.origin=10
        self.xg=150
        self.yg=150
        self.update=True

def UpdateShape(x ):
    p.update = True

def union(a,b):
  x = min(a[0], b[0])
  y = min(a[1], b[1])
  w = max(a[0]+a[2], b[0]+b[2]) - x
  h = max(a[1]+a[3], b[1]+b[3]) - y
  return (x, y, w, h)

def intersection(a,b):
  x = max(a[0], b[0])
  y = max(a[1], b[1])
  w = min(a[0]+a[2], b[0]+b[2]) - x
  h = min(a[1]+a[3], b[1]+b[3]) - y
  if w<0 or h<0: return () # or (0,0,0,0) ?
  return (x, y, w, h)

def NoisyPolygon(pRef,n):
#    vector<Point> c
    p = pRef;
#    vector<vector<Point> > contour;
    p = p+n*np.random.random_sample((p.shape[0],p.shape[1]))-n/2.0
    if (n==0):
        return p
    c = np.empty(shape=[0, 2])
    minX = p[0][0]
    maxX = p[0][0]
    minY = p[0][1]
    maxY = p[0][1]
    for i in range( 0,p.shape[0]):
        next = i + 1;
        if (next == p.shape[0]):
            next = 0;
        u = p[next] - p[i]
        d = int(cv.norm(u))
        a = np.arctan2(u[1], u[0])
        step = 1
        if (n != 0):
            step = d // n
        for j in range( 1,int(d),int(max(step, 1))):
            while  True:
                pAct = (u*j) / (d)
                r = n*np.random.random_sample()
                theta = a + 2*math.pi*np.random.random_sample()
#                pNew = Point(Point2d(r*cos(theta) + pAct.x + p[i].x, r*sin(theta) + pAct.y + p[i].y));
                pNew = np.array([(r*np.cos(theta) + pAct[0] + p[i][0], r*np.sin(theta) + pAct[1] + p[i][1])])
                if (pNew[0][0]>=0 and pNew[0][1]>=0):
                    break
            if (pNew[0][0]<minX):
                minX = pNew[0][0]
            if (pNew[0][0]>maxX):
                maxX = pNew[0][0]
            if (pNew[0][1]<minY):
                minY = pNew[0][1]
            if (pNew[0][1]>maxY):
                maxY = pNew[0][1]
            c = np.append(c,pNew,axis = 0)
    return c

#static vector<Point> NoisyPolygon(vector<Point> pRef, double n);
#static void UpdateShape(int , void *r);
#static void AddSlider(String sliderName, String windowName, int minSlider, int maxSlider, int valDefault, int *valSlider, void(*f)(int, void *), void *r);
def AddSlider(sliderName,windowName,minSlider,maxSlider,valDefault, update):
    cv.createTrackbar(sliderName, windowName, valDefault,maxSlider-minSlider+1, update)
    cv.setTrackbarMin(sliderName, windowName, minSlider)
    cv.setTrackbarMax(sliderName, windowName, maxSlider)
    cv.setTrackbarPos(sliderName, windowName, valDefault)

#    vector<Point> ctrRef;
#    vector<Point> ctrRotate, ctrNoisy, ctrNoisyRotate, ctrNoisyRotateShift;
#    // build a shape with 5 vertex
ctrRef = np.array([(250,250),(400, 250),(400, 300),(250, 300),(180, 270)])
cg = np.mean(ctrRef,axis=0)
p=ThParameters()
cv.namedWindow("FD Curve matching");
# A rotation with center at (150,150) of angle 45 degrees and a scaling of 5/10
AddSlider("Noise", "FD Curve matching", 0, 20, p.levelNoise,  UpdateShape)
AddSlider("Angle", "FD Curve matching", 0, 359, p.angle,  UpdateShape)
AddSlider("Scale", "FD Curve matching", 5, 100, p.scale10, UpdateShape)
AddSlider("Origin", "FD Curve matching", 0, 100, p.origin, UpdateShape)
AddSlider("Xg", "FD Curve matching", 150, 450, p.xg, UpdateShape)
AddSlider("Yg", "FD Curve matching", 150, 450, p.yg, UpdateShape)
code = 0
img = np.zeros((300,512,3), np.uint8)
print ("******************** PRESS g TO MATCH CURVES *************\n")

while (code!=27):
    code = cv.waitKey(60)
    if p.update:
        p.levelNoise=cv.getTrackbarPos('Noise','FD Curve matching')
        p.angle=cv.getTrackbarPos('Angle','FD Curve matching')
        p.scale10=cv.getTrackbarPos('Scale','FD Curve matching')
        p.origin=cv.getTrackbarPos('Origin','FD Curve matching')
        p.xg=cv.getTrackbarPos('Xg','FD Curve matching')
        p.yg=cv.getTrackbarPos('Yg','FD Curve matching')

        r = cv.getRotationMatrix2D((p.xg, p.yg), angle=p.angle, scale=10.0/ p.scale10);
        ctrNoisy= NoisyPolygon(ctrRef,p.levelNoise)
        ctrNoisy1 = np.reshape(ctrNoisy,(ctrNoisy.shape[0],1,2))
        ctrNoisyRotate = cv.transform(ctrNoisy1,r)
        ctrNoisyRotateShift = np.empty([ctrNoisyRotate.shape[0],1,2],dtype=np.int32)
        for  i in range(0,ctrNoisy.shape[0]):
            k=(i+(p.origin*ctrNoisy.shape[0])//100)% ctrNoisyRotate.shape[0]
            ctrNoisyRotateShift[i] = ctrNoisyRotate[k]
#       To draw contour using drawcontours
        cc= np.reshape(ctrNoisyRotateShift,[ctrNoisyRotateShift.shape[0],2])
        c = [ ctrRef,cc]
        p.update = False;
        rglobal =(0,0,0,0)
        for i in range(0,2):
            r = cv.boundingRect(c[i])
            rglobal = union(rglobal,r)
        r = list(rglobal)
        r[2] = r[2]+10
        r[3] = r[3]+10
        rglobal = tuple(r)
        img = np.zeros((2 * rglobal[3], 2 * rglobal[2], 3), np.uint8)
        cv.drawContours(img, c, 0, (255,0,0),1);
        cv.drawContours(img, c, 1, (0, 255, 0),1);
        cv.circle(img, tuple(c[0][0]), 5, (255, 0, 0),3);
        cv.circle(img, tuple(c[1][0]), 5, (0, 255, 0),3);
        cv.imshow("FD Curve matching", img);
    if code == ord('d') :
        cv.destroyWindow("FD Curve matching");
        cv.namedWindow("FD Curve matching");
# A rotation with center at (150,150) of angle 45 degrees and a scaling of 5/10
        AddSlider("Noise", "FD Curve matching", 0, 20, p.levelNoise,  UpdateShape)
        AddSlider("Angle", "FD Curve matching", 0, 359, p.angle,  UpdateShape)
        AddSlider("Scale", "FD Curve matching", 5, 100, p.scale10,  UpdateShape)
        AddSlider("Origin%%", "FD Curve matching", 0, 100, p.origin, UpdateShape)
        AddSlider("Xg", "FD Curve matching", 150, 450, p.xg,  UpdateShape)
        AddSlider("Yg", "FD Curve matching", 150, 450, p.yg,  UpdateShape)
    if  code == ord('g'):
        fit = cv.ximgproc.createContourFitting(1024,16);
# sampling contour we want 256 points
        cn= np.reshape(ctrRef,[ctrRef.shape[0],1,2])

        ctrRef2d = cv.ximgproc.contourSampling(cn,  256)
        ctrRot2d = cv.ximgproc.contourSampling(ctrNoisyRotateShift,  256)
        fit.setFDSize(16)
        c1 = ctrRef2d
        c2 = ctrRot2d
        alphaPhiST, dist	 = fit.estimateTransformation(ctrRot2d, ctrRef2d)
        print( "Transform *********\n Origin = ", 1-alphaPhiST[0,0] ," expected ", p.origin / 100. ,"\n")
        print( "Angle = ", alphaPhiST[0,1] * 180 / math.pi ," expected " , p.angle,"\n")
        print( "Scale = " ,alphaPhiST[0,2] ," expected " , p.scale10 / 10.0 , "\n")
        dst = cv.ximgproc.transformFD(ctrRot2d, alphaPhiST,cn, False);
        ctmp= np.reshape(dst,[dst.shape[0],2])
        cdst=ctmp.astype(int)

        c = [ ctrRef,cc,cdst]
        cv.drawContours(img, c, 2, (0,0,255),1);
        cv.circle(img, (int(c[2][0][0]),int(c[2][0][1])), 5, (0, 0, 255),5);
        cv.imshow("FD Curve matching", img);



#!/usr/bin/env python

# This file is part of OpenCV project.
# It is subject to the license terms in the LICENSE file found in the top-level directory
# of this distribution and at http://opencv.org/license.html
# Copyright (C) 2020 by Archit Rungta

from __future__ import unicode_literals # Needed for python2

import hdr_parser, sys, re, os
from string import Template
from pprint import pprint
from collections import namedtuple

import os, shutil

if sys.version_info[0] >= 3:
    from io import StringIO
else:
    from cStringIO import StringIO


from parse_tree import *

mod_template = ""
with open("binding_templates_cpp/cv_core.cpp", "r") as f:
    mod_template = Template(f.read())


def normalize_name(name):
    return name.replace('.', '::')

def normalize_class_name(name):
    _, classes, name = split_decl_name(normalize_name(name))
    return "_".join(classes+[name])

def normalize_full_name(name):
    ns, classes, name = split_decl_name(normalize_name(name))
    return "::".join(ns)+'::'+'_'.join(classes+[name])



def split_decl_name(name):
    chunks = name.split('::')
    namespace = chunks[:-1]
    classes = []
    while namespace and '::'.join(namespace) not in namespaces:
        classes.insert(0, namespace.pop())

    ns = '::'.join(namespace)
    if ns not in namespaces and ns:
        assert(0)

    return namespace, classes, chunks[-1]

def registered_tp_search(tp):
    found = False
    if not tp:
        return True
    for tpx in registered_types:
        if re.findall(tpx, tp):
            found = True
            break
    return found

namespaces = {}
enums = []
classes = {}
functions = {}
registered_types = ["int", "Size.*", "Rect.*", "Scalar", "RotatedRect", "Point.*", "explicit", "string", "bool", "uchar",
                    "Vec.*", "float", "double", "char", "Mat", "size_t", "RNG", "TermCriteria"]

class ClassInfo(ClassInfo):
    def get_cpp_code_header(self):
        if self.ismap:
            return 'mod.map_type<%s>("%s");\n'%(self.name, self.mapped_name)
        if not self.base:
            return 'mod.add_type<%s>("%s");\n' % (self.name, self.mapped_name)
        else:
            return 'mod.add_type<%s>("%s", jlcxx::julia_base_type<%s>());\n' % (self.name, self.mapped_name, self.base)

    def get_cpp_code_body(self):
        if self.ismap:
            return ''
        cpp_code = StringIO()
        for cons in self.constructors:
            cons.__class__ = FuncVariant
            cpp_code.write(cons.get_cons_code(self.name, self.mapped_name))
        #add get/set
        cpp_code.write('\n')
        cpp_code.write(self.get_setters())
        cpp_code.write('\n')
        cpp_code.write(self.get_getters())
        cpp_code.write(';')
        return cpp_code.getvalue()

        # return code for functions and setters and getters if simple class or functions and map type

    def get_prop_func_cpp(self, mode, propname):
        return "jlopencv_" + self.mapped_name + "_"+mode+"_"+propname

    def get_getters(self):
        stra = ""
        for prop in self.props:
            if not self.isalgorithm:
                stra = stra + '\nmod.method("%s", [](const %s &cobj) {return %scobj.%s;});' % (self.get_prop_func_cpp("get", prop.name), self.name, '(int)' if prop.tp in enums else '', prop.name)
            else:
                stra = stra + '\nmod.method("%s", [](const cv::Ptr<%s> &cobj) {return %scobj->%s;});' % (self.get_prop_func_cpp("get", prop.name), self.name,'(int)' if prop.tp in enums else '',  prop.name)
        return stra

    def get_setters(self):
        stra = ""
        for prop in self.props:
            if prop.readonly:
                continue
            if not self.isalgorithm:
                stra = stra + '\nmod.method("%s", [](%s &cobj,const force_enum_int<%s>::Type &v) {cobj.%s=(%s)v;});' % (self.get_prop_func_cpp("set", prop.name), self.name, prop.tp, prop.name, prop.tp)
            else:
                stra = stra + '\nmod.method("%s", [](cv::Ptr<%s> cobj, const force_enum_int<%s>::Type &v) {cobj->%s=(%s)v;});' % (self.get_prop_func_cpp("set", prop.name), self.name, prop.tp, prop.name, prop.tp)
        return stra

class FuncVariant(FuncVariant):

    def get_return(self):
        outstr = ""
        for arg in self.inlist+self.optlist:
            if arg.tp not in pass_by_val_types and arg.tp not in enums and self.promote_type(arg.tp)!=arg.tp:
                outstr = outstr + "%s=%s_down;\n"%(arg.name, arg.name)

        if len(self.outlist)==0:
            return outstr+";"
        elif len(self.outlist)==1:
            return outstr+"return %s;" % ( ('(int64_t)' if self.outlist[0].tp in enums else ('' if self.promote_type(self.outlist[0].tp)==self.outlist[0].tp else '(%s)'%self.promote_type(self.outlist[0].tp))) + self.outlist[0].name)
        return outstr+"return make_tuple(%s);" %  ",".join(["move(%s)" %  (('(int64_t)' if x.tp in enums else  ('' if self.promote_type(x.tp)==x.tp else '(%s)'%self.promote_type(x.tp))) +x.name) for x in self.outlist])

    def promote_type(self, tp):
        if tp=='int':
            return 'long long'
        elif tp =='float':
            return 'double'
        return tp

    def get_argument(self, isalgo):
        args = self.inlist + self.optlist
        if self.classname!="" and not self.isconstructor and not self.isstatic:
            if isalgo:
                args = [ArgInfo("cobj", ("cv::Ptr<%s>" % self.classname))] + args
            else:
                args = [ArgInfo("cobj", self.classname)] + args

        argnamelist = []
        for arg in args:
            if arg.tp in pass_by_val_types:
                print("PATHWAY NOT TESTED")
                argnamelist.append(arg.tp[:-1] +"& "+arg.name)
            elif arg.tp in enums:
                argnamelist.append("int64_t& " + arg.name)
            else:
                if arg.tp=='bool':
                    # Bool pass-by-reference is broken
                    argnamelist.append(arg.tp+" " +arg.name)
                else:
                    argnamelist.append(self.promote_type(arg.tp) + "& "+arg.name)
        # argnamelist = [(arg.tp if arg.tp not in pass_by_val_types else arg.tp[:-1]) +"& "+arg.name for arg in args]
        argstr = ", ".join(argnamelist)
        return argstr

    def get_def_outtypes(self):
        outstr = ""
        for arg in self.deflist:
            outstr = outstr + "%s %s;"%(arg.tp if arg.tp not in pass_by_val_types else arg.tp[:-1], arg.name)
        for arg in self.inlist+self.optlist:
            if arg.tp not in pass_by_val_types and arg.tp not in enums and self.promote_type(arg.tp)!=arg.tp:
                outstr = outstr + "%s %s_down=(%s)%s;"%(arg.tp if arg.tp not in pass_by_val_types else arg.tp[:-1], arg.name, arg.tp, arg.name)

        return outstr

    def get_retval(self, isalgo):
        if self.rettype:
            stra = "auto retval = "
        else:
            stra = ""
        arlist = []
        for x in self.args:
            if x.tp in pass_by_val_types:
                arlist.append("&"+x.name)
            elif x.tp in enums:
                arlist.append("(%s)%s" %(x.tp, x.name))
            else:
                if self.promote_type(x.tp) == x.tp:
                    arlist.append(x.name)
                else:
                    if len([y for y in self.inlist+self.optlist if y.name==x.name])>0:
                    # print("ss")
                        arlist.append("%s_down" %(x.name))
                    else:
                        arlist.append(x.name)

        argstr = ", ".join(arlist)
        if self.classname and not self.isstatic:
            stra = stra + "cobj%s%s(%s); " %("->" if isalgo else ".",self.name.split('::')[-1], argstr)
        else:
            stra = stra + "%s(%s);" % (self.name, argstr)
        return stra

    def get_cons_code(self, name, mapped_name):
        # if self.get_argument(False) == '':
        #     return ''
        arglist = []
        for x in self.args:
            if x.tp in pass_by_val_types:
                arglist.append("&"+x.name)
            elif x.tp in enums:
                arglist.append("(%s)%s" %(x.tp, x.name))
            else:
                if self.promote_type(x.tp) == x.tp:
                    arglist.append(x.name)
                else:
                    # print("ss")
                    arglist.append("%s_down" %(x.name))
        return 'mod.method("%s", [](%s) { %s return jlcxx::create<%s>(%s);});' % (self.get_wrapper_name(), self.get_argument(False), self.get_def_outtypes(), name, " ,".join(arglist))

    def get_complete_code(self, classname, isalgo=False):
        outstr = '.method("%s",  [](%s) {%s %s %s})' % (self.get_wrapper_name(), self.get_argument(isalgo),self.get_def_outtypes(), self.get_retval(isalgo), self.get_return())
        return outstr



def gen(srcfiles):
    namespaces, default_values = gen_tree(srcfiles)
    cpp_code = StringIO()
    include_code = StringIO()
    nsi = sorted(namespaces.items(), key =lambda x: x[0])

    for name, ns in nsi:
        cpp_code.write("using namespace %s;\n" % name.replace(".", "::"))

        if name.split('.')[-1] == '':
            continue
        nsname = name
        nsprefix = '_'.join(nsname.split('::')[1:])

        def sort_classes(classes):
            class_inherits = []
            class_inherits_names = set()
            class_noinherits = []
            parent = {}
            for name, cl in classes:
                if cl.base:
                    class_inherits.append((name, cl))
                    parent[name] = cl.base
                    class_inherits_names.add(name)
                else:
                    class_noinherits.append((name,cl))

            final_order = class_noinherits

            while len(class_inherits)>0:
                for cli in class_inherits:
                    if parent[cli[0]] not in class_inherits_names:
                        final_order.append(cli)
                        class_inherits.remove(cli)
                        class_inherits_names.remove(cli[0])


            return final_order

        sorted_cls = sort_classes(ns.classes.items())
        for name, cl in sorted_cls:
            cl.__class__ = ClassInfo
            cpp_code.write(cl.get_cpp_code_header())
            if cl.base:
                include_code.write("""
template <>
struct SuperType<%s>
{
    typedef %s type;
};
                                    """ % (cl.name.replace('.', '::'), cl.base.replace('.', '::')))

        for e1,e2 in ns.enums.items():
            # cpp_code.write('\n    mod.add_bits<{0}>("{1}", jlcxx::julia_type("CppEnum"));'.format(e2[0], e2[1]))
            enums.append(e2[0])
            enums.append(e2[1])
            enums.append(e2[0].replace("cv::", "").replace("::", '_'))


        for tp in ns.register_types:
            cpp_code.write('   mod.add_type<%s>("%s");\n' %(tp, normalize_class_name(tp)))

    # print(enums)
    for name, ns in namespaces.items():

        nsname = name.replace("::", "_")
        for name, cl in ns.classes.items():
            cl.__class__ = ClassInfo
            cpp_code.write(cl.get_cpp_code_body())
            for mname, fs in cl.methods.items():
                for f in fs:
                    f.__class__ = FuncVariant
                    cpp_code.write('\n    mod%s;'  % f.get_complete_code(cl.name, cl.isalgorithm))
            # for f in cl.constructors:
            #     cpp_code.write('\n    %s; \n'  % f.get_cons_code(cl.name, cl.mapped_name))


        for mname, fs in ns.funcs.items():
            for f in fs:
                f.__class__ = FuncVariant
                cpp_code.write('\n    mod%s;' % f.get_complete_code("", False))

        for mapname, name in sorted(ns.consts.items()):
            cpp_code.write('    mod.set_const("%s_%s", (force_enum_int<decltype(%s)>::Type)%s);\n'%(nsname, name, mapname, mapname))
            compat_name = re.sub(r"([a-z])([A-Z])", r"\1_\2", name).upper()
            if name != compat_name:
                cpp_code.write('    mod.set_const("%s_%s", (force_enum_int<decltype(%s)>::Type)%s);\n'%(nsname, compat_name, mapname, mapname))
    default_values = list(set(default_values))
    for val in default_values:
        # val = handle_cpp_arg(val)

        cpp_code.write('    mod.method("%s", [](){return (force_enum_int<decltype(%s)>::Type)%s;});\n'%(get_var(val), val, val))

    with open ('autogen_cpp/cv_core.cpp', 'w') as fd:
        fd.write(mod_template.substitute(include_code = include_code.getvalue(), cpp_code=cpp_code.getvalue()))



srcfiles = hdr_parser.opencv_hdr_list
if len(sys.argv) > 1:
    srcfiles = [l.strip() for l in sys.argv[1].split(';')]


gen(srcfiles)



#!/usr/bin/env python
import os
import cv2 as cv
import numpy as np

from tests_common import NewOpenCVTests, unittest

class cudabgsegm_test(NewOpenCVTests):
    def setUp(self):
        super(cudabgsegm_test, self).setUp()
        if not cv.cuda.getCudaEnabledDeviceCount():
            self.skipTest("No CUDA-capable device is detected")

    def test_cudabgsegm(self):
        lr = 0.05
        sz = (128,128,1)
        npMat = (np.random.random(sz) * 255).astype(np.uint8)
        cuMat = cv.cuda_GpuMat(npMat)
        cuMatBg = cv.cuda_GpuMat(cuMat.size(),cuMat.type())
        cuMatFg = cv.cuda_GpuMat(cuMat.size(),cuMat.type())

        mog = cv.cuda.createBackgroundSubtractorMOG()
        mog.apply(cuMat, lr, cv.cuda.Stream_Null(), cuMatFg)
        mog.getBackgroundImage(cv.cuda.Stream_Null(),cuMatBg)
        self.assertTrue(sz[:2] == cuMatFg.size() == cuMatBg.size())
        self.assertTrue(sz[2] == cuMatFg.channels() == cuMatBg.channels())
        self.assertTrue(cv.CV_8UC1 == cuMatFg.type() == cuMatBg.type())
        mog = cv.cuda.createBackgroundSubtractorMOG()
        self.assertTrue(np.allclose(cuMatFg.download(),mog.apply(cuMat, lr, cv.cuda.Stream_Null()).download()))
        self.assertTrue(np.allclose(cuMatBg.download(),mog.getBackgroundImage(cv.cuda.Stream_Null()).download()))

        mog2 = cv.cuda.createBackgroundSubtractorMOG2()
        mog2.apply(cuMat, lr, cv.cuda.Stream_Null(), cuMatFg)
        mog2.getBackgroundImage(cv.cuda.Stream_Null(),cuMatBg)
        self.assertTrue(sz[:2] == cuMatFg.size() == cuMatBg.size())
        self.assertTrue(sz[2] == cuMatFg.channels() == cuMatBg.channels())
        self.assertTrue(cv.CV_8UC1 == cuMatFg.type() == cuMatBg.type())
        mog2 = cv.cuda.createBackgroundSubtractorMOG2()
        self.assertTrue(np.allclose(cuMatFg.download(),mog2.apply(cuMat, lr, cv.cuda.Stream_Null()).download()))
        self.assertTrue(np.allclose(cuMatBg.download(),mog2.getBackgroundImage(cv.cuda.Stream_Null()).download()))

if __name__ == '__main__':
    NewOpenCVTests.bootstrap()


#!/usr/bin/env python
import os
import numpy as np
import cv2 as cv

from tests_common import NewOpenCVTests
#from unittest import TestCase as NewOpenCVTests


class ovis_contrib_test(NewOpenCVTests):

    def setUp(self):
        super().setUp()
        # use software rendering
        os.environ["OPENCV_OVIS_RENDERSYSTEM"] = "Tiny Rendering Subsystem"
        # in case something goes wrong
        os.environ["OPENCV_OVIS_VERBOSE_LOG"] = "1"

    def test_multiWindow(self):
        win0 = cv.ovis.createWindow("main", (1, 1))
        win1 = cv.ovis.createWindow("other", (1, 1))
        del win1
        win1 = cv.ovis.createWindow("other", (1, 1))
        del win1

    def test_addResourceLocation(self):
        win0 = cv.ovis.createWindow("main", (1, 1))
        with self.assertRaises(cv.error):
            # must be called before the first createWindow
            cv.ovis.addResourceLocation(".")

    def test_texStride(self):
        win = cv.ovis.createWindow("main", (1, 1))
        data = np.zeros((200, 200), dtype=np.uint8)
        cv.ovis.createPlaneMesh("plane", (1, 1), data[50:-50, 50:-50])


if __name__ == '__main__':
    NewOpenCVTests.bootstrap()



#!/usr/bin/env python
import os
import cv2 as cv
import numpy as np

from tests_common import NewOpenCVTests, unittest

def create_affine_transform_matrix(size,angle):
    return np.array([[np.cos(angle), -np.sin(angle), size[1]/2], [np.sin(angle), np.cos(angle), 0]])
def create_perspective_transform_matrix(size,angle):
    return np.vstack([create_affine_transform_matrix(size,angle),[0, 0, 1]])

class cudawarping_test(NewOpenCVTests):
    def setUp(self):
        super(cudawarping_test, self).setUp()
        if not cv.cuda.getCudaEnabledDeviceCount():
            self.skipTest("No CUDA-capable device is detected")

    def test_resize(self):
        dstSz = (256,256)
        interp = cv.INTER_NEAREST
        npMat = (np.random.random((128,128,3))*255).astype(np.uint8)

        cuMat = cv.cuda_GpuMat(npMat)
        cuMatDst = cv.cuda_GpuMat(dstSz,cuMat.type())

        self.assertTrue(np.allclose(cv.cuda.resize(cuMat,dstSz,interpolation=interp).download(),
            cv.resize(npMat,dstSz,interpolation=interp)))

        cv.cuda.resize(cuMat,dstSz,cuMatDst,interpolation=interp)
        self.assertTrue(np.allclose(cuMatDst.download(),cv.resize(npMat,dstSz,interpolation=interp)))

    def test_warp(self):
        npMat = (np.random.random((128,128,3))*255).astype(np.uint8)
        size = npMat.shape[:2]
        M1 = create_affine_transform_matrix(size,np.pi/2)

        cuMat = cv.cuda_GpuMat(npMat)
        cuMatDst = cv.cuda_GpuMat(size,cuMat.type())

        borderType = cv.BORDER_REFLECT101
        self.assertTrue(np.allclose(cv.cuda.warpAffine(cuMat,M1,size,borderMode=borderType).download(),
            cv.warpAffine(npMat,M1,size, borderMode=borderType)))
        cv.cuda.warpAffine(cuMat,M1,size,cuMatDst,borderMode=borderType)
        self.assertTrue(np.allclose(cuMatDst.download(),cv.warpAffine(npMat,M1,size,borderMode=borderType)))

        interpolation = cv.INTER_NEAREST
        flags = interpolation | cv.WARP_INVERSE_MAP
        dst_gold = cv.warpAffine(npMat, M1, size, flags = flags)
        cuMaps = cv.cuda.buildWarpAffineMaps(M1,True,size)
        dst = cv.remap(npMat, cuMaps[0].download(), cuMaps[1].download(),interpolation)
        self.assertTrue(np.allclose(dst,dst_gold))

        xmap = cv.cuda_GpuMat(size,cv.CV_32FC1)
        ymap = cv.cuda_GpuMat(size,cv.CV_32FC1)
        cv.cuda.buildWarpAffineMaps(M1,True,size,xmap,ymap)
        dst = cv.remap(npMat, xmap.download(), ymap.download(),interpolation)
        self.assertTrue(np.allclose(dst,dst_gold))

        M2 = create_perspective_transform_matrix(size,np.pi/2)
        np.allclose(cv.cuda.warpPerspective(cuMat,M2,size,borderMode=borderType).download(),
                    cv.warpPerspective(npMat,M2,size,borderMode=borderType))
        cv.cuda.warpPerspective(cuMat,M2,size,cuMatDst,borderMode=borderType)
        self.assertTrue(np.allclose(cuMatDst.download(),cv.warpPerspective(npMat,M2,size,borderMode=borderType)))

        dst_gold = cv.warpPerspective(npMat, M2, size, flags = flags)
        cuMaps = cv.cuda.buildWarpPerspectiveMaps(M2,True,size)
        dst = cv.remap(npMat, cuMaps[0].download(), cuMaps[1].download(),interpolation)
        self.assertTrue(np.allclose(dst,dst_gold))

        cv.cuda.buildWarpPerspectiveMaps(M2,True,size,xmap,ymap)
        dst = cv.remap(npMat, xmap.download(), ymap.download(),interpolation)
        self.assertTrue(np.allclose(dst,dst_gold))

if __name__ == '__main__':
    NewOpenCVTests.bootstrap()


#!/usr/bin/env python

def substitute(build, output_dir):

    # setup the template engine
    template_dir = os.path.join(os.path.dirname(__file__), 'templates')
    jtemplate    = Environment(loader=FileSystemLoader(template_dir), trim_blocks=True, lstrip_blocks=True)

    # add the filters
    jtemplate.filters['csv'] = csv
    jtemplate.filters['stripExtraSpaces'] = stripExtraSpaces

    # load the template
    template = jtemplate.get_template('template_build_info.m')

    # create the build directory
    output_dir  = output_dir+'/+cv'
    if not os.path.isdir(output_dir):
      os.mkdir(output_dir)

    # populate template
    populated = template.render(build=build, time=time)
    with open(os.path.join(output_dir, 'buildInformation.m'), 'wb') as f:
        f.write(populated.encode('utf-8'))

if __name__ == "__main__":
    """
    Usage: python build_info.py
                                --os os_version_string
                                --arch [bitness processor]
                                --compiler [id version]
                                --mex_arch arch_string
                                --mex_script /path/to/mex/script
                                --cxx_flags [-list -of -flags -to -passthrough]
                                --opencv_version version_string
                                --commit commit_hash_if_using_git
                                --modules [core imgproc highgui etc]
                                --configuration Debug/Release
                                --outdir /path/to/write/build/info

    build_info.py generates a Matlab function that can be invoked with a call to
      >> cv.buildInformation();

    This function prints a summary of the user's OS, OpenCV and Matlab build
    given the information passed to this module. build_info.py invokes Jinja2
    on the template_build_info.m template.
    """

    # parse the input options
    import sys, re, os, time
    from argparse import ArgumentParser
    parser = ArgumentParser()
    parser.add_argument('--os')
    parser.add_argument('--arch', nargs=2)
    parser.add_argument('--compiler', nargs='+')
    parser.add_argument('--mex_arch')
    parser.add_argument('--mex_script')
    parser.add_argument('--mex_opts', default=['-largeArrayDims'], nargs='*')
    parser.add_argument('--cxx_flags', default=[], nargs='*')
    parser.add_argument('--opencv_version', default='', nargs='?')
    parser.add_argument('--commit', default='Not in working git tree', nargs='?')
    parser.add_argument('--modules', nargs='+')
    parser.add_argument('--configuration')
    parser.add_argument('--outdir')
    build = parser.parse_args()

    from filters import *
    from jinja2 import Environment, FileSystemLoader

    # populate the build info template
    substitute(build, build.outdir)



#!/usr/bin/env python

'''
This example shows the functionalities of lines extraction finished by LSDDetector class.

USAGE: lsd_lines_extraction.py [<path_to_input_image>]
'''

import sys
import cv2 as cv

if __name__ == '__main__':
    print(__doc__)

    if len(sys.argv) > 1:
        fname = sys.argv[1]
    else :
        fname = '../data/corridor.jpg'

    img = cv.imread(fname)

    if img is None:
        print('Failed to load image file:', fname)
        sys.exit(1)

    gray = cv.cvtColor(img, cv.COLOR_BGR2GRAY)

    lsd = cv.line_descriptor_LSDDetector.createLSDDetector()

    lines = lsd.detect(gray, 2, 1)
    for kl in lines:
        if kl.octave == 0:
            # cv.line only accepts integer coordinate
            pt1 = (int(kl.startPointX), int(kl.startPointY))
            pt2 = (int(kl.endPointX), int(kl.endPointY))
            cv.line(img, pt1, pt2, [255, 0, 0], 2)

    cv.imshow('output', img)
    cv.waitKey(0)
    cv.destroyAllWindows()



#!/usr/bin/env python
import os
import cv2 as cv
import numpy as np

from tests_common import NewOpenCVTests, unittest

class cudaarithm_test(NewOpenCVTests):
    def setUp(self):
        super(cudaarithm_test, self).setUp()
        if not cv.cuda.getCudaEnabledDeviceCount():
            self.skipTest("No CUDA-capable device is detected")

    def test_cudaarithm(self):
        npMat = (np.random.random((128, 128, 3)) * 255).astype(np.uint8)

        cuMat = cv.cuda_GpuMat(npMat)
        cuMatDst = cv.cuda_GpuMat(cuMat.size(),cuMat.type())
        cuMatB = cv.cuda_GpuMat(cuMat.size(),cv.CV_8UC1)
        cuMatG = cv.cuda_GpuMat(cuMat.size(),cv.CV_8UC1)
        cuMatR = cv.cuda_GpuMat(cuMat.size(),cv.CV_8UC1)

        self.assertTrue(np.allclose(cv.cuda.merge(cv.cuda.split(cuMat)),npMat))

        cv.cuda.split(cuMat,[cuMatB,cuMatG,cuMatR])
        cv.cuda.merge([cuMatB,cuMatG,cuMatR],cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),npMat))

        shift = (np.random.random((cuMat.channels(),)) * 8).astype(np.uint8).tolist()
        self.assertTrue(np.allclose(cv.cuda.rshift(cuMat,shift).download(),npMat  >> shift))
        cv.cuda.rshift(cuMat,shift,cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),npMat >> shift))

        self.assertTrue(np.allclose(cv.cuda.lshift(cuMat,shift).download(),(npMat << shift).astype('uint8')))
        cv.cuda.lshift(cuMat,shift,cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),(npMat << shift).astype('uint8')))

    def test_arithmetic(self):
        npMat1 = np.random.random((128, 128, 3)) - 0.5
        npMat2 = np.random.random((128, 128, 3)) - 0.5

        cuMat1 = cv.cuda_GpuMat()
        cuMat2 = cv.cuda_GpuMat()
        cuMat1.upload(npMat1)
        cuMat2.upload(npMat2)
        cuMatDst = cv.cuda_GpuMat(cuMat1.size(),cuMat1.type())

        self.assertTrue(np.allclose(cv.cuda.add(cuMat1, cuMat2).download(),
                                         cv.add(npMat1, npMat2)))

        cv.cuda.add(cuMat1, cuMat2, cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),cv.add(npMat1, npMat2)))

        self.assertTrue(np.allclose(cv.cuda.subtract(cuMat1, cuMat2).download(),
                                         cv.subtract(npMat1, npMat2)))

        cv.cuda.subtract(cuMat1, cuMat2, cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),cv.subtract(npMat1, npMat2)))

        self.assertTrue(np.allclose(cv.cuda.multiply(cuMat1, cuMat2).download(),
                                         cv.multiply(npMat1, npMat2)))

        cv.cuda.multiply(cuMat1, cuMat2, cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),cv.multiply(npMat1, npMat2)))

        self.assertTrue(np.allclose(cv.cuda.divide(cuMat1, cuMat2).download(),
                                         cv.divide(npMat1, npMat2)))

        cv.cuda.divide(cuMat1, cuMat2, cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),cv.divide(npMat1, npMat2)))

        self.assertTrue(np.allclose(cv.cuda.absdiff(cuMat1, cuMat2).download(),
                                         cv.absdiff(npMat1, npMat2)))

        cv.cuda.absdiff(cuMat1, cuMat2, cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),cv.absdiff(npMat1, npMat2)))

        self.assertTrue(np.allclose(cv.cuda.compare(cuMat1, cuMat2, cv.CMP_GE).download(),
                                         cv.compare(npMat1, npMat2, cv.CMP_GE)))

        cuMatDst1 = cv.cuda_GpuMat(cuMat1.size(),cv.CV_8UC3)
        cv.cuda.compare(cuMat1, cuMat2, cv.CMP_GE, cuMatDst1)
        self.assertTrue(np.allclose(cuMatDst1.download(),cv.compare(npMat1, npMat2, cv.CMP_GE)))

        self.assertTrue(np.allclose(cv.cuda.abs(cuMat1).download(),
                                         np.abs(npMat1)))

        cv.cuda.abs(cuMat1, cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),np.abs(npMat1)))

        self.assertTrue(np.allclose(cv.cuda.sqrt(cv.cuda.sqr(cuMat1)).download(),
                                    cv.cuda.abs(cuMat1).download()))

        cv.cuda.sqr(cuMat1, cuMatDst)
        cv.cuda.sqrt(cuMatDst, cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),cv.cuda.abs(cuMat1).download()))

        self.assertTrue(np.allclose(cv.cuda.log(cv.cuda.exp(cuMat1)).download(),
                                                            npMat1))

        cv.cuda.exp(cuMat1, cuMatDst)
        cv.cuda.log(cuMatDst, cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),npMat1))

        self.assertTrue(np.allclose(cv.cuda.pow(cuMat1, 2).download(),
                                         cv.pow(npMat1, 2)))

        cv.cuda.pow(cuMat1, 2, cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),cv.pow(npMat1, 2)))

    def test_logical(self):
        npMat1 = (np.random.random((128, 128)) * 255).astype(np.uint8)
        npMat2 = (np.random.random((128, 128)) * 255).astype(np.uint8)

        cuMat1 = cv.cuda_GpuMat()
        cuMat2 = cv.cuda_GpuMat()
        cuMat1.upload(npMat1)
        cuMat2.upload(npMat2)
        cuMatDst = cv.cuda_GpuMat(cuMat1.size(),cuMat1.type())

        self.assertTrue(np.allclose(cv.cuda.bitwise_or(cuMat1, cuMat2).download(),
                                         cv.bitwise_or(npMat1, npMat2)))

        cv.cuda.bitwise_or(cuMat1, cuMat2, cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),cv.bitwise_or(npMat1, npMat2)))

        self.assertTrue(np.allclose(cv.cuda.bitwise_and(cuMat1, cuMat2).download(),
                                         cv.bitwise_and(npMat1, npMat2)))

        cv.cuda.bitwise_and(cuMat1, cuMat2, cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),cv.bitwise_and(npMat1, npMat2)))

        self.assertTrue(np.allclose(cv.cuda.bitwise_xor(cuMat1, cuMat2).download(),
                                         cv.bitwise_xor(npMat1, npMat2)))

        cv.cuda.bitwise_xor(cuMat1, cuMat2, cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),cv.bitwise_xor(npMat1, npMat2)))

        self.assertTrue(np.allclose(cv.cuda.bitwise_not(cuMat1).download(),
                                         cv.bitwise_not(npMat1)))

        cv.cuda.bitwise_not(cuMat1, cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),cv.bitwise_not(npMat1)))

        self.assertTrue(np.allclose(cv.cuda.min(cuMat1, cuMat2).download(),
                                         cv.min(npMat1, npMat2)))

        cv.cuda.min(cuMat1, cuMat2, cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),cv.min(npMat1, npMat2)))

        self.assertTrue(np.allclose(cv.cuda.max(cuMat1, cuMat2).download(),
                                         cv.max(npMat1, npMat2)))

        cv.cuda.max(cuMat1, cuMat2, cuMatDst)
        self.assertTrue(np.allclose(cuMatDst.download(),cv.max(npMat1, npMat2)))

        self.assertTrue(cv.cuda.minMax(cuMat1),cv.minMaxLoc(npMat1)[:2])
        self.assertTrue(cv.cuda.minMaxLoc(cuMat1),cv.minMaxLoc(npMat1))

    def test_convolution(self):
        npMat = (np.random.random((128, 128)) * 255).astype(np.float32)
        npDims = np.array(npMat.shape)
        kernel = (np.random.random((3, 3)) * 1).astype(np.float32)
        kernelDims = np.array(kernel.shape)
        iS = (kernelDims/2).astype(int)
        iE = npDims - kernelDims + iS

        cuMat = cv.cuda_GpuMat(npMat)
        cuKernel= cv.cuda_GpuMat(kernel)
        cuMatDst = cv.cuda_GpuMat(tuple(npDims - kernelDims + 1), cuMat.type())
        conv = cv.cuda.createConvolution()

        self.assertTrue(np.allclose(conv.convolve(cuMat,cuKernel,ccorr=True).download(),
                    cv.filter2D(npMat,-1,kernel,anchor=(-1,-1))[iS[0]:iE[0]+1,iS[1]:iE[1]+1]))

        conv.convolve(cuMat,cuKernel,cuMatDst,True)
        self.assertTrue(np.allclose(cuMatDst.download(),
                    cv.filter2D(npMat,-1,kernel,anchor=(-1,-1))[iS[0]:iE[0]+1,iS[1]:iE[1]+1]))

    def test_inrange(self):
        npMat = (np.random.random((128, 128, 3)) * 255).astype(np.float32)

        bound1 = np.random.random((4,)) * 255
        bound2 = np.random.random((4,)) * 255
        lowerb = np.minimum(bound1, bound2).tolist()
        upperb = np.maximum(bound1, bound2).tolist()

        cuMat = cv.cuda_GpuMat()
        cuMat.upload(npMat)

        self.assertTrue((cv.cuda.inRange(cuMat, lowerb, upperb).download() ==
                         cv.inRange(npMat, np.array(lowerb), np.array(upperb))).all())

        cuMatDst = cv.cuda_GpuMat(cuMat.size(), cv.CV_8UC1)
        cv.cuda.inRange(cuMat, lowerb, upperb, cuMatDst)
        self.assertTrue((cuMatDst.download() ==
                         cv.inRange(npMat, np.array(lowerb), np.array(upperb))).all())

if __name__ == '__main__':
    NewOpenCVTests.bootstrap()



#!/usr/bin/env python3

# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright (c) 2019, Josh Chien. All rights reserved.

from argparse import ArgumentParser
import numpy as np
from PIL import Image
import io
import warnings
import os
import cairo
from cairosvg import svg2png
import math
import tempfile

def SaveArucoDictBytesList(filePath = "arucoDictBytesList.npz"):
    import numpy as np

    # cv2 is optional dependency
    try:
        import cv2
        from cv2 import aruco

        # Name, Flag
        dictInfo = \
        [
            ("DICT_4X4_1000", aruco.DICT_4X4_1000),
            ("DICT_5X5_1000", aruco.DICT_5X5_1000),
            ("DICT_6X6_1000", aruco.DICT_6X6_1000),
            ("DICT_7X7_1000", aruco.DICT_7X7_1000),
            ("DICT_ARUCO_ORIGINAL", aruco.DICT_ARUCO_ORIGINAL),
            ("DICT_APRILTAG_16h5", aruco.DICT_APRILTAG_16h5),
            ("DICT_APRILTAG_25h9", aruco.DICT_APRILTAG_25h9),
            ("DICT_APRILTAG_36h10", aruco.DICT_APRILTAG_36h10),
            ("DICT_APRILTAG_36h11", aruco.DICT_APRILTAG_36h11),
        ]

        arucoDictBytesList = {}
        for name, flag in dictInfo:
            arucoDict = aruco.Dictionary_get(flag)
            arucoDictBytesList[name] = arucoDict.bytesList

        np.savez_compressed(filePath, **arucoDictBytesList)
        return arucoDictBytesList

    except Exception as e:
        warnings.warn(str(e))
        return None

    return None

class MarkerPrinter:

    debugMode = None # "LINE" "BLOCK"

    # Static Vars
    # SVG https://oreillymedia.github.io/Using_SVG/guide/units.html
    # for PDF and SVG, 1 pixel = 1/72 inch, 1 cm = 1/2.54 inch, 1pixl = 2.54/72 cm, 1cm = 72/2.54 pixels
    ptPerMeter = 72 / 2.54 * 100

    surface = {
            ".SVG": cairo.SVGSurface,
            ".PDF": cairo.PDFSurface,
            ".PS": cairo.PSSurface }

    if (os.path.isfile("arucoDictBytesList.npz")):
        arucoDictBytesList = np.load("arucoDictBytesList.npz")
    else:
        warnings.warn("Missing build-in arucoDictBytesList.npz, generate it again")
        arucoDictBytesList = SaveArucoDictBytesList(filePath = "arucoDictBytesList.npz")

    arucoDictMarkerSize = \
        {
            "DICT_4X4_1000": 4,
            "DICT_5X5_1000": 5,
            "DICT_6X6_1000": 6,
            "DICT_7X7_1000": 7,
            "DICT_ARUCO_ORIGINAL": 5,
            "DICT_APRILTAG_16h5": 4,
            "DICT_APRILTAG_25h9": 5,
            "DICT_APRILTAG_36h10": 6,
            "DICT_APRILTAG_36h11": 6,
        }

    def ArucoBits(dictionary, markerID):
        bytesList = MarkerPrinter.arucoDictBytesList[dictionary][markerID].ravel()
        markerSize = MarkerPrinter.arucoDictMarkerSize[dictionary]

        arucoBits = np.zeros(shape = (markerSize, markerSize), dtype = bool)
        base2List = np.array( [128, 64, 32, 16, 8, 4, 2, 1], dtype = np.uint8)
        currentByteIdx = 0
        currentByte = bytesList[currentByteIdx]
        currentBit = 0
        for row in range(markerSize):
            for col in range(markerSize):
                if(currentByte >= base2List[currentBit]):
                    arucoBits[row, col] = True
                    currentByte -= base2List[currentBit]
                currentBit = currentBit + 1
                if(currentBit == 8):
                    currentByteIdx = currentByteIdx + 1
                    currentByte = bytesList[currentByteIdx]
                    if(8 * (currentByteIdx + 1) > arucoBits.size):
                        currentBit = 8 * (currentByteIdx + 1) - arucoBits.size
                    else:
                        currentBit = 0;
        return arucoBits

    def __DrawBlock(context,
        dictionary = None, markerLength = None, borderBits = 1,
        chessboardSize = (1, 1), squareLength = None, firstMarkerID = 0,
        blockX = 0, blockY = 0, originX = 0, originY = 0, pageBorderX = 0, pageBorderY = 0,
        mode = "CHESS" ):

        if(squareLength is None):
            squareLength = markerLength

        if(markerLength is None):
            markerLength = squareLength

        if((squareLength is None) or (markerLength is None)):
            raise ValueError("lenght is None")

        dawMarkerBlock = False
        if ((mode == "ARUCO") or (mode == "ARUCOGRID")):
            dawMarkerBlock = True
        elif(chessboardSize[1] % 2 == 0):
            dawMarkerBlock = (( blockX % 2 == 0 ) == ( blockY % 2 == 0 ))
        else:
            dawMarkerBlock = (( blockX % 2 == 0 ) != ( blockY % 2 == 0 ))

        if(dawMarkerBlock):
            if (mode != "CHESS"):
                if(dictionary is None):
                    raise ValueError("dictionary is None")

                if (mode == "CHARUCO"):
                    originX = (blockX - originX) * squareLength + (squareLength - markerLength)*0.5 + pageBorderX
                    originY = (blockY - originY) * squareLength + (squareLength - markerLength)*0.5 + pageBorderY
                else:
                    originX = (blockX - originX) * squareLength + pageBorderX
                    originY = (blockY - originY) * squareLength + pageBorderY

                context.set_source_rgba(0.0, 0.0, 0.0, 1.0)
                context.rectangle(originX, originY, markerLength, markerLength)
                context.fill()

                # Generate marker
                if  (mode == "CHARUCO"):
                    markerID = firstMarkerID + (blockY * chessboardSize[0] + blockX) // 2
                elif (mode == "ARUCO"):
                    markerID = firstMarkerID
                elif (mode == "ARUCOGRID"):
                    markerID = firstMarkerID + (blockY * chessboardSize[0] + blockX)

                marker = MarkerPrinter.ArucoBits(dictionary, markerID)
                markerSize = marker.shape[0]
                unitLength = markerLength / (float)(markerSize + borderBits * 2)

                markerBitMap = np.zeros(shape = (markerSize+borderBits*2, markerSize+borderBits*2), dtype = bool)
                markerBitMap[borderBits:-borderBits,borderBits:-borderBits] = marker
                markerBitMap = np.swapaxes(markerBitMap, 0, 1)

                # Compute edges
                hEdges = np.zeros(shape = (markerSize+1,markerSize+1), dtype = bool)
                vEdges = np.zeros(shape = (markerSize+1,markerSize+1), dtype = bool)

                for mx in range(markerSize):
                    for my in range(markerSize+1):
                        if ( markerBitMap[mx + borderBits, my + borderBits - 1] ^ markerBitMap[mx + borderBits, my + borderBits]):
                            hEdges[mx, my] = True

                for mx in range(markerSize+1):
                    for my in range(markerSize):
                        if ( markerBitMap[mx + borderBits - 1, my + borderBits] ^ markerBitMap[mx + borderBits, my + borderBits]):
                            vEdges[mx, my] = True

                # Use for debug, check edge or position is correct or not
                if(MarkerPrinter.debugMode is not None):
                    if(MarkerPrinter.debugMode.upper() == "LINE"):
                        context.set_source_rgba(1.0, 1.0, 1.0, 1.0)
                        context.set_line_width(unitLength * 0.1)
                        for mx in range(markerSize+1):
                            for my in range(markerSize+1):
                                if(hEdges[mx, my]):
                                    context.move_to(originX + unitLength * (mx + borderBits    ), originY + unitLength * (my + borderBits    ))
                                    context.line_to(originX + unitLength * (mx + borderBits + 1), originY + unitLength * (my + borderBits    ))
                                    context.stroke()
                                if(vEdges[mx, my]):
                                    context.move_to(originX + unitLength * (mx + borderBits    ), originY + unitLength * (my + borderBits    ))
                                    context.line_to(originX + unitLength * (mx + borderBits    ), originY + unitLength * (my + borderBits + 1))
                                    context.stroke()

                    elif(MarkerPrinter.debugMode.upper() == "BLOCK"):
                        context.set_source_rgba(1.0, 1.0, 1.0, 1.0)
                        for mx in range(markerSize):
                            for my in range(markerSize):
                                if(markerBitMap[mx + borderBits, my + borderBits]):
                                    context.rectangle(
                                        originX + unitLength * (mx + borderBits),
                                        originY + unitLength * (my + borderBits),
                                        unitLength, unitLength)
                                    context.fill()

                else:
                    while(True):
                        found = False

                        # Find start position
                        sx = 0
                        sy = 0
                        for my in range(markerSize):
                            for mx in range(markerSize):
                                if(hEdges[mx, my]):
                                    found = True
                                    sx = mx
                                    sy = my
                                    if(markerBitMap[sx + borderBits, sy + borderBits - 1]):
                                        context.set_source_rgba(0.0, 0.0, 0.0, 1.0)
                                    else:
                                        context.set_source_rgba(1.0, 1.0, 1.0, 1.0)
                                    break
                            if(found):
                                break

                        context.move_to (originX + unitLength * (sx + borderBits), originY + unitLength * (sy + borderBits))

                        # Use wall follower maze solving algorithm to draw white part
                        cx = sx
                        cy = sy
                        cd = 3 # 0 right, 1 down, 2 left, 3 up
                        while(True):
                            nd = (cd + 1)%4
                            moved = False
                            if(nd == 0):
                                if(hEdges[cx, cy]):
                                    hEdges[cx, cy] = False
                                    cx = cx + 1
                                    moved = True
                            elif(nd == 1):
                                if(vEdges[cx, cy]):
                                    vEdges[cx, cy] = False
                                    cy = cy + 1
                                    moved = True
                            elif(nd == 2):
                                if(hEdges[cx - 1, cy]):
                                    hEdges[cx - 1, cy] = False
                                    cx = cx - 1
                                    moved = True
                            elif(nd == 3):
                                if(vEdges[cx, cy - 1]):
                                    vEdges[cx, cy - 1] = False
                                    cy = cy - 1
                                    moved = True

                            if((cx == sx) and (cy == sy)):
                                context.close_path ()
                                break
                            else:
                                if(moved):
                                    context.line_to(originX + unitLength * (cx + borderBits), originY + unitLength * (cy + borderBits))
                                cd = nd

                        if (found):
                            context.fill()
                        else:
                            break

        else:
            originX = (blockX - originX) * squareLength + pageBorderX
            originY = (blockY - originY) * squareLength + pageBorderY
            context.set_source_rgba(0.0, 0.0, 0.0, 1.0)
            context.rectangle(originX, originY, squareLength, squareLength)
            context.fill()

    def __CheckChessMarkerImage(chessboardSize, squareLength, subSize=None, pageBorder=(0,0)):
        if(len(chessboardSize) != 2):
            raise ValueError("len(chessboardSize) != 2")
        else:
            sizeX, sizeY = chessboardSize

        if(len(pageBorder) != 2):
            raise ValueError("len(pageBorder) != 2")
        else:
            pageBorderX, pageBorderY = pageBorder

        if(sizeX <= 1):
            raise ValueError("sizeX <= 1")

        if(sizeY <= 1):
            raise ValueError("sizeY <= 1")

        if(squareLength <= 0):
            raise ValueError("squareLength <= 0")

        if(pageBorderX < 0):
            raise ValueError("pageBorderX < 0")

        if(pageBorderY < 0):
            raise ValueError("pageBorderY < 0")

        if(subSize is not None):
            subSizeX, subSizeY = subSize

            if(subSizeX < 0):
                raise ValueError("subSizeX < 0")

            if(subSizeY < 0):
                raise ValueError("subSizeY < 0")

    def PreviewChessMarkerImage(chessboardSize, squareLength, pageBorder=(0, 0), dpi=96):
        MarkerPrinter.__CheckChessMarkerImage(chessboardSize, squareLength, pageBorder=pageBorder)

        squareLength = squareLength * MarkerPrinter.ptPerMeter
        pageBorder = (pageBorder[0] * MarkerPrinter.ptPerMeter, pageBorder[1] * MarkerPrinter.ptPerMeter)

        prevImage = None
        with tempfile.TemporaryDirectory() as tmpdirname:
            with MarkerPrinter.surface[".SVG"] (
                os.path.join(tmpdirname, "tempSVG.svg"),
                chessboardSize[0] * squareLength + pageBorder[0] * 2,
                chessboardSize[1] * squareLength + pageBorder[1] * 2) as surface:
                context = cairo.Context(surface)

                context.set_source_rgba(0.5, 0.5, 0.5, 1.0)
                context.rectangle(0, 0,
                    chessboardSize[0] * squareLength + pageBorder[0] * 2,
                    chessboardSize[1] * squareLength + pageBorder[1] * 2)
                context.fill()

                context.set_source_rgba(1.0, 1.0, 1.0, 1.0)
                context.rectangle(pageBorder[0], pageBorder[1],
                    chessboardSize[0] * squareLength,
                    chessboardSize[1] * squareLength)
                context.fill()

                for bx in range(chessboardSize[0]):
                    for by in range(chessboardSize[1]):
                        MarkerPrinter.__DrawBlock(
                            context = context,
                            chessboardSize = chessboardSize,
                            squareLength = squareLength,
                            blockX = bx,
                            blockY = by,
                            pageBorderX = pageBorder[0],
                            pageBorderY = pageBorder[1],
                            mode = "CHESS")

            with open(os.path.join(tmpdirname, "tempSVG.svg")) as file:
                prevImage = Image.open(io.BytesIO(svg2png(bytestring=file.read(), dpi=dpi)))

        return prevImage

    def GenChessMarkerImage(filePath, chessboardSize, squareLength, subSize=None, pageBorder=(0, 0)):
        MarkerPrinter.__CheckChessMarkerImage(chessboardSize, squareLength, subSize=subSize, pageBorder=pageBorder)

        squareLength = squareLength * MarkerPrinter.ptPerMeter
        pageBorder = (pageBorder[0] * MarkerPrinter.ptPerMeter, pageBorder[1] * MarkerPrinter.ptPerMeter)

        # Check
        path, nameExt = os.path.split(filePath)
        name, ext = os.path.splitext(nameExt)

        if(len(path) > 0):
            if not(os.path.isdir(path)):
                os.makedirs(path)

        if((ext.upper() != ".SVG") and (ext.upper() != ".PS") and (ext.upper() != ".PDF")):
            raise ValueError("file extention is not supported, should be: svg, ps, pdf")

        # Draw
        with MarkerPrinter.surface[ext.upper()] (
            filePath,
            chessboardSize[0] * squareLength + pageBorder[0] * 2,
            chessboardSize[1] * squareLength + pageBorder[1] * 2) as surface:
            context = cairo.Context(surface)

            context.set_source_rgba(0.5, 0.5, 0.5, 1.0)
            context.rectangle(0, 0,
                chessboardSize[0] * squareLength + pageBorder[0] * 2,
                chessboardSize[1] * squareLength + pageBorder[1] * 2)
            context.fill()

            context.set_source_rgba(1.0, 1.0, 1.0, 1.0)
            context.rectangle(pageBorder[0], pageBorder[1],
                chessboardSize[0] * squareLength,
                chessboardSize[1] * squareLength)
            context.fill()

            for bx in range(chessboardSize[0]):
                for by in range(chessboardSize[1]):
                    MarkerPrinter.__DrawBlock(
                        context = context,
                        chessboardSize = chessboardSize,
                        squareLength = squareLength,
                        blockX = bx,
                        blockY = by,
                        pageBorderX = pageBorder[0],
                        pageBorderY = pageBorder[1],
                        mode = "CHESS" )

        if(subSize is not None):
            subDivide = (\
                chessboardSize[0] // subSize[0] + int(chessboardSize[0] % subSize[0] > 0),
                chessboardSize[1] // subSize[1] + int(chessboardSize[1] % subSize[1] > 0))

            subChessboardBlockX = np.clip ( np.arange(0, subSize[0] * subDivide[0] + 1, subSize[0]), 0, chessboardSize[0])
            subChessboardBlockY = np.clip ( np.arange(0, subSize[1] * subDivide[1] + 1, subSize[1]), 0, chessboardSize[1])

            subChessboardSliceX = subChessboardBlockX.astype(np.float) * squareLength
            subChessboardSliceY = subChessboardBlockY.astype(np.float) * squareLength

            for subXID in range(subDivide[0]):
                for subYID in range(subDivide[1]):
                    subName = name + \
                        "_X" + str(subChessboardBlockX[subXID]) + "_" + str(subChessboardBlockX[subXID+1]) + \
                        "_Y" + str(subChessboardBlockY[subYID]) + "_" + str(subChessboardBlockY[subYID+1])

                    with MarkerPrinter.surface[ext.upper()](
                        os.path.join(path, subName + ext),
                        subChessboardSliceX[subXID+1] - subChessboardSliceX[subXID] + pageBorder[0] * 2,
                        subChessboardSliceY[subYID+1] - subChessboardSliceY[subYID] + pageBorder[1] * 2) as surface:
                        context = cairo.Context(surface)

                        context.set_source_rgba(0.5, 0.5, 0.5, 1.0)
                        context.rectangle(0, 0,
                            subChessboardSliceX[subXID+1] - subChessboardSliceX[subXID] + pageBorder[0] * 2,
                            subChessboardSliceY[subYID+1] - subChessboardSliceY[subYID] + pageBorder[1] * 2)
                        context.fill()

                        context.set_source_rgba(1.0, 1.0, 1.0, 1.0)
                        context.rectangle(pageBorder[0], pageBorder[1],
                            subChessboardSliceX[subXID+1] - subChessboardSliceX[subXID],
                            subChessboardSliceY[subYID+1] - subChessboardSliceY[subYID])
                        context.fill()

                        for bx in range(subChessboardBlockX[subXID+1] - subChessboardBlockX[subXID]):
                            for by in range(subChessboardBlockY[subYID+1] - subChessboardBlockY[subYID]):
                                MarkerPrinter.__DrawBlock(
                                    context = context,
                                    chessboardSize = chessboardSize,
                                    squareLength = squareLength,
                                    blockX = subChessboardBlockX[subXID] + bx,
                                    blockY = subChessboardBlockY[subYID] + by,
                                    originX = subChessboardBlockX[subXID],
                                    originY = subChessboardBlockY[subYID],
                                    pageBorderX = pageBorder[0],
                                    pageBorderY = pageBorder[1],
                                    mode = "CHESS" )


    def __CheckArucoMarkerImage(dictionary, markerID, markerLength, borderBits=1, pageBorder=(0, 0)):
        if(len(pageBorder) != 2):
            raise ValueError("len(pageBorder) != 2")
        else:
            pageBorderX, pageBorderY = pageBorder

        if not (dictionary in MarkerPrinter.arucoDictBytesList):
            raise ValueError("dictionary is not support")

        if(MarkerPrinter.arucoDictBytesList[dictionary].shape[0] <= markerID ):
            raise ValueError("markerID is not in aruce dictionary")

        if(markerID < 0):
            raise ValueError("markerID < 0")

        if(markerLength <= 0):
            raise ValueError("markerLength <= 0")

        if(borderBits <= 0):
            raise ValueError("borderBits <= 0")

        if(pageBorderX < 0):
            raise ValueError("pageBorderX < 0")

        if(pageBorderY < 0):
            raise ValueError("pageBorderY < 0")

    def PreviewArucoMarkerImage(dictionary, markerID, markerLength, borderBits=1, pageBorder=(0, 0), dpi=96):
        MarkerPrinter.__CheckArucoMarkerImage(dictionary, markerID, markerLength, borderBits=borderBits, pageBorder=pageBorder)

        markerLength = markerLength * MarkerPrinter.ptPerMeter
        pageBorder = (pageBorder[0] * MarkerPrinter.ptPerMeter, pageBorder[1] * MarkerPrinter.ptPerMeter)

        prevImage = None
        with tempfile.TemporaryDirectory() as tmpdirname:
            with MarkerPrinter.surface[".SVG"] (
                os.path.join(tmpdirname, "tempSVG.svg"),
                markerLength + pageBorder[0] * 2,
                markerLength + pageBorder[1] * 2) as surface:
                context = cairo.Context(surface)

                context.set_source_rgba(0.5, 0.5, 0.5, 1.0)
                context.rectangle(0, 0,
                    markerLength + pageBorder[0] * 2,
                    markerLength + pageBorder[1] * 2)
                context.fill()

                context.set_source_rgba(1.0, 1.0, 1.0, 1.0)
                context.rectangle(pageBorder[0], pageBorder[1],
                    markerLength,
                    markerLength)
                context.fill()

                MarkerPrinter.__DrawBlock(
                    context = context,
                    dictionary = dictionary,
                    markerLength = markerLength,
                    borderBits = borderBits,
                    firstMarkerID = markerID,
                    pageBorderX = pageBorder[0],
                    pageBorderY = pageBorder[1],
                    mode = "ARUCO")

            with open(os.path.join(tmpdirname, "tempSVG.svg")) as file:
                prevImage = Image.open(io.BytesIO(svg2png(bytestring=file.read(), dpi=dpi)))

        return prevImage

    def GenArucoMarkerImage(filePath, dictionary, markerID, markerLength, borderBits=1, pageBorder=(0, 0)):
        MarkerPrinter.__CheckArucoMarkerImage(dictionary, markerID, markerLength, borderBits=borderBits, pageBorder=pageBorder)

        markerLength = markerLength * MarkerPrinter.ptPerMeter
        pageBorder = (pageBorder[0] * MarkerPrinter.ptPerMeter, pageBorder[1] * MarkerPrinter.ptPerMeter)

        # Check
        path, nameExt = os.path.split(filePath)
        name, ext = os.path.splitext(nameExt)

        if(len(path) > 0):
            if not(os.path.isdir(path)):
                os.makedirs(path)

        if((ext.upper() != ".SVG") and (ext.upper() != ".PS") and (ext.upper() != ".PDF")):
            raise ValueError("file extention is not supported, should be: svg, ps, pdf")

        # Draw
        with MarkerPrinter.surface[ext.upper()] (
            filePath,
            markerLength + pageBorder[0] * 2,
            markerLength + pageBorder[1] * 2) as surface:
            context = cairo.Context(surface)

            context.set_source_rgba(0.5, 0.5, 0.5, 1.0)
            context.rectangle(0, 0,
                markerLength + pageBorder[0] * 2,
                markerLength + pageBorder[1] * 2)
            context.fill()

            context.set_source_rgba(1.0, 1.0, 1.0, 1.0)
            context.rectangle(pageBorder[0], pageBorder[1],
                markerLength,
                markerLength)
            context.fill()

            MarkerPrinter.__DrawBlock(
                context = context,
                dictionary = dictionary,
                markerLength = markerLength,
                borderBits = borderBits,
                firstMarkerID = markerID,
                pageBorderX = pageBorder[0],
                pageBorderY = pageBorder[1],
                mode = "ARUCO")

    def __CheckCharucoMarkerImage(dictionary, chessboardSize, squareLength, markerLength, borderBits=1, subSize=None, pageBorder=(0, 0)):
        if(len(chessboardSize) != 2):
            raise ValueError("len(chessboardSize) != 2")
        else:
            sizeX, sizeY = chessboardSize

        if(len(pageBorder) != 2):
            raise ValueError("len(pageBorder) != 2")
        else:
            pageBorderX, pageBorderY = pageBorder

        if not (dictionary in MarkerPrinter.arucoDictBytesList):
            raise ValueError("dictionary is not support")

        if(MarkerPrinter.arucoDictBytesList[dictionary].shape[0] < (( sizeX * sizeY ) // 2)):
            raise ValueError("aruce dictionary is not enough for your board size")

        if(sizeX <= 1):
            raise ValueError("sizeX <= 1")

        if(sizeY <= 1):
            raise ValueError("sizeY <= 1")

        if(squareLength <= 0):
            raise ValueError("squareLength <= 0")

        if(markerLength <= 0):
            raise ValueError("markerLength <= 0")

        if(squareLength < markerLength):
            raise ValueError("squareLength < markerLength")

        if(borderBits <= 0):
            raise ValueError("borderBits <= 0")

        if(pageBorderX < 0):
            raise ValueError("pageBorderX < 0")

        if(pageBorderY < 0):
            raise ValueError("pageBorderY < 0")

        if(subSize is not None):
            subSizeX, subSizeY = subSize

            if(subSizeX < 0):
                raise ValueError("subSizeX < 0")

            if(subSizeY < 0):
                raise ValueError("subSizeY < 0")

    def PreviewCharucoMarkerImage(dictionary, chessboardSize, squareLength, markerLength, borderBits=1, pageBorder=(0, 0), dpi=96):
        MarkerPrinter.__CheckCharucoMarkerImage(dictionary, chessboardSize, squareLength, markerLength, borderBits=borderBits, pageBorder=pageBorder)

        squareLength = squareLength * MarkerPrinter.ptPerMeter
        markerLength = markerLength * MarkerPrinter.ptPerMeter
        pageBorder = (pageBorder[0] * MarkerPrinter.ptPerMeter, pageBorder[1] * MarkerPrinter.ptPerMeter)

        prevImage = None
        with tempfile.TemporaryDirectory() as tmpdirname:
            with MarkerPrinter.surface[".SVG"] (
                os.path.join(tmpdirname, "tempSVG.svg"),
                chessboardSize[0] * squareLength + pageBorder[0] * 2,
                chessboardSize[1] * squareLength + pageBorder[1] * 2) as surface:
                context = cairo.Context(surface)

                context.set_source_rgba(0.5, 0.5, 0.5, 1.0)
                context.rectangle(0, 0,
                    chessboardSize[0] * squareLength + pageBorder[0] * 2,
                    chessboardSize[1] * squareLength + pageBorder[1] * 2)
                context.fill()

                context.set_source_rgba(1.0, 1.0, 1.0, 1.0)
                context.rectangle(pageBorder[0], pageBorder[1],
                    chessboardSize[0] * squareLength,
                    chessboardSize[1] * squareLength)
                context.fill()

                for bx in range(chessboardSize[0]):
                    for by in range(chessboardSize[1]):
                        MarkerPrinter.__DrawBlock(
                            context = context,
                            dictionary = dictionary,
                            markerLength = markerLength,
                            borderBits = borderBits,
                            chessboardSize = chessboardSize,
                            squareLength = squareLength,
                            blockX = bx,
                            blockY = by,
                            pageBorderX = pageBorder[0],
                            pageBorderY = pageBorder[1],
                            mode = "CHARUCO")

            with open(os.path.join(tmpdirname, "tempSVG.svg")) as file:
                prevImage = Image.open(io.BytesIO(svg2png(bytestring=file.read(), dpi=dpi)))

        return prevImage

    def GenCharucoMarkerImage(filePath, dictionary, chessboardSize, squareLength, markerLength, borderBits=1, subSize=None, pageBorder=(0, 0)):
        MarkerPrinter.__CheckCharucoMarkerImage(dictionary, chessboardSize, squareLength, markerLength, borderBits=borderBits, subSize=subSize, pageBorder=pageBorder)

        squareLength = squareLength * MarkerPrinter.ptPerMeter
        markerLength = markerLength * MarkerPrinter.ptPerMeter
        pageBorder = (pageBorder[0] * MarkerPrinter.ptPerMeter, pageBorder[1] * MarkerPrinter.ptPerMeter)

        # Check
        path, nameExt = os.path.split(filePath)
        name, ext = os.path.splitext(nameExt)

        if(len(path) > 0):
            if not(os.path.isdir(path)):
                os.makedirs(path)

        if((ext.upper() != ".SVG") and (ext.upper() != ".PS") and (ext.upper() != ".PDF")):
            raise ValueError("file extention is not supported, should be: svg, ps, pdf")

        # Draw
        with MarkerPrinter.surface[ext.upper()] (
            filePath,
            chessboardSize[0] * squareLength + pageBorder[0] * 2,
            chessboardSize[1] * squareLength + pageBorder[1] * 2) as surface:
            context = cairo.Context(surface)

            context.set_source_rgba(0.5, 0.5, 0.5, 1.0)
            context.rectangle(0, 0,
                chessboardSize[0] * squareLength + pageBorder[0] * 2,
                chessboardSize[1] * squareLength + pageBorder[1] * 2)
            context.fill()

            context.set_source_rgba(1.0, 1.0, 1.0, 1.0)
            context.rectangle(pageBorder[0], pageBorder[1],
                chessboardSize[0] * squareLength,
                chessboardSize[1] * squareLength)
            context.fill()

            for bx in range(chessboardSize[0]):
                for by in range(chessboardSize[1]):
                    MarkerPrinter.__DrawBlock(
                        context = context,
                        dictionary = dictionary,
                        markerLength = markerLength,
                        borderBits = borderBits,
                        chessboardSize = chessboardSize,
                        squareLength = squareLength,
                        blockX = bx,
                        blockY = by,
                        pageBorderX = pageBorder[0],
                        pageBorderY = pageBorder[1],
                        mode = "CHARUCO")

        if(subSize is not None):
            subDivide = (\
                chessboardSize[0] // subSize[0] + int(chessboardSize[0] % subSize[0] > 0),
                chessboardSize[1] // subSize[1] + int(chessboardSize[1] % subSize[1] > 0))

            subChessboardBlockX = np.clip ( np.arange(0, subSize[0] * subDivide[0] + 1, subSize[0]), 0, chessboardSize[0])
            subChessboardBlockY = np.clip ( np.arange(0, subSize[1] * subDivide[1] + 1, subSize[1]), 0, chessboardSize[1])

            subChessboardSliceX = subChessboardBlockX.astype(np.float) * squareLength
            subChessboardSliceY = subChessboardBlockY.astype(np.float) * squareLength

            for subXID in range(subDivide[0]):
                for subYID in range(subDivide[1]):
                    subName = name + \
                        "_X" + str(subChessboardBlockX[subXID]) + "_" + str(subChessboardBlockX[subXID+1]) + \
                        "_Y" + str(subChessboardBlockY[subYID]) + "_" + str(subChessboardBlockY[subYID+1])

                    with MarkerPrinter.surface[ext.upper()](
                        os.path.join(path, subName + ext),
                        subChessboardSliceX[subXID+1] - subChessboardSliceX[subXID] + pageBorder[0] * 2,
                        subChessboardSliceY[subYID+1] - subChessboardSliceY[subYID] + pageBorder[1] * 2) as surface:
                        context = cairo.Context(surface)

                        context.set_source_rgba(0.5, 0.5, 0.5, 1.0)
                        context.rectangle(0, 0,
                            subChessboardSliceX[subXID+1] - subChessboardSliceX[subXID] + pageBorder[0] * 2,
                            subChessboardSliceY[subYID+1] - subChessboardSliceY[subYID] + pageBorder[1] * 2)
                        context.fill()

                        context.set_source_rgba(1.0, 1.0, 1.0, 1.0)
                        context.rectangle(pageBorder[0], pageBorder[1],
                            subChessboardSliceX[subXID+1] - subChessboardSliceX[subXID],
                            subChessboardSliceY[subYID+1] - subChessboardSliceY[subYID])
                        context.fill()

                        for bx in range(subChessboardBlockX[subXID+1] - subChessboardBlockX[subXID]):
                            for by in range(subChessboardBlockY[subYID+1] - subChessboardBlockY[subYID]):
                                MarkerPrinter.__DrawBlock(
                                    context = context,
                                    dictionary = dictionary,
                                    markerLength = markerLength,
                                    borderBits = borderBits,
                                    chessboardSize = chessboardSize,
                                    squareLength = squareLength,
                                    blockX = subChessboardBlockX[subXID] + bx,
                                    blockY = subChessboardBlockY[subYID] + by,
                                    originX = subChessboardBlockX[subXID],
                                    originY = subChessboardBlockY[subYID],
                                    pageBorderX = pageBorder[0],
                                    pageBorderY = pageBorder[1],
                                    mode = "CHARUCO")

    def __CheckArucoGridMarkerImage(dictionary, chessboardSize, markerLength, markerSeparation, firstMarker, borderBits=1, subSize=None, pageBorder=(0, 0)):
        if(len(chessboardSize) != 2):
            raise ValueError("len(chessboardSize) != 2")
        else:
            sizeX, sizeY = chessboardSize

        if(len(pageBorder) != 2):
            raise ValueError("len(pageBorder) != 2")
        else:
            pageBorderX, pageBorderY = pageBorder

        if not (dictionary in MarkerPrinter.arucoDictBytesList):
            raise ValueError("dictionary is not support")

        if(MarkerPrinter.arucoDictBytesList[dictionary].shape[0] < (( sizeX * sizeY ) + firstMarker)):
            raise ValueError("aruce dictionary is not enough for your board size and firstMarker")

        if(sizeX <= 1):
            raise ValueError("sizeX <= 1")

        if(sizeY <= 1):
            raise ValueError("sizeY <= 1")

        if(markerLength <= 0):
            raise ValueError("markerLength <= 0")

        if(markerSeparation <= 0):
            raise ValueError("markerSeparation <= 0")

        if(borderBits <= 0):
            raise ValueError("borderBits <= 0")

        if(pageBorderX < 0):
            raise ValueError("pageBorderX < 0")

        if(pageBorderY < 0):
            raise ValueError("pageBorderY < 0")

        if(subSize is not None):
            subSizeX, subSizeY = subSize

            if(subSizeX < 0):
                raise ValueError("subSizeX < 0")

            if(subSizeY < 0):
                raise ValueError("subSizeY < 0")

    def PreviewArucoGridMarkerImage(dictionary, chessboardSize, markerLength, markerSeparation, firstMarker, borderBits=1, pageBorder=(0, 0), dpi=96):
        MarkerPrinter.__CheckArucoGridMarkerImage(dictionary, chessboardSize, markerLength, markerSeparation, firstMarker, borderBits=borderBits, pageBorder=pageBorder)

        markerLength = markerLength * MarkerPrinter.ptPerMeter
        markerSeparation = markerSeparation * MarkerPrinter.ptPerMeter
        pageBorder = (pageBorder[0] * MarkerPrinter.ptPerMeter, pageBorder[1] * MarkerPrinter.ptPerMeter)

        prevImage = None
        with tempfile.TemporaryDirectory() as tmpdirname:
            with MarkerPrinter.surface[".SVG"] (
                os.path.join(tmpdirname, "tempSVG.svg"),
                chessboardSize[0] * markerLength + (chessboardSize[0] - 1) * markerSeparation + pageBorder[0] * 2,
                chessboardSize[1] * markerLength + (chessboardSize[1] - 1) * markerSeparation + pageBorder[1] * 2) as surface:
                context = cairo.Context(surface)

                context.set_source_rgba(0.5, 0.5, 0.5, 1.0)
                context.rectangle(0, 0,
                    chessboardSize[0] * markerLength + (chessboardSize[0] - 1) * markerSeparation + pageBorder[0] * 2,
                    chessboardSize[1] * markerLength + (chessboardSize[1] - 1) * markerSeparation + pageBorder[1] * 2)
                context.fill()

                context.set_source_rgba(1.0, 1.0, 1.0, 1.0)
                context.rectangle(pageBorder[0], pageBorder[1],
                    chessboardSize[0] * markerLength + (chessboardSize[0] - 1) * markerSeparation,
                    chessboardSize[1] * markerLength + (chessboardSize[1] - 1) * markerSeparation)
                context.fill()

                for bx in range(chessboardSize[0]):
                    for by in range(chessboardSize[1]):
                        MarkerPrinter.__DrawBlock(
                            context = context,
                            dictionary = dictionary,
                            markerLength = markerLength,
                            borderBits = borderBits,
                            chessboardSize = chessboardSize,
                            squareLength = markerLength + markerSeparation,
                            firstMarkerID = firstMarker,
                            blockX = bx,
                            blockY = by,
                            pageBorderX = pageBorder[0],
                            pageBorderY = pageBorder[1],
                            mode = "ARUCOGRID")

            with open(os.path.join(tmpdirname, "tempSVG.svg")) as file:
                prevImage = Image.open(io.BytesIO(svg2png(bytestring=file.read(), dpi=dpi)))

        return prevImage

    def GenArucoGridMarkerImage(filePath, dictionary, chessboardSize, markerLength, markerSeparation, firstMarker, borderBits=1, subSize=None, pageBorder=(0, 0)):
        MarkerPrinter.__CheckArucoGridMarkerImage(dictionary, chessboardSize, markerLength, markerSeparation, firstMarker, borderBits=borderBits, subSize=subSize, pageBorder=pageBorder)

        markerLength = markerLength * MarkerPrinter.ptPerMeter
        markerSeparation = markerSeparation * MarkerPrinter.ptPerMeter
        pageBorder = (pageBorder[0] * MarkerPrinter.ptPerMeter, pageBorder[1] * MarkerPrinter.ptPerMeter)

        # Check
        path, nameExt = os.path.split(filePath)
        name, ext = os.path.splitext(nameExt)

        if(len(path) > 0):
            if not(os.path.isdir(path)):
                os.makedirs(path)

        if((ext.upper() != ".SVG") and (ext.upper() != ".PS") and (ext.upper() != ".PDF")):
            raise ValueError("file extention is not supported, should be: svg, ps, pdf")

        # Draw
        with MarkerPrinter.surface[ext.upper()] (
            filePath,
            chessboardSize[0] * markerLength + (chessboardSize[0] - 1) * markerSeparation + pageBorder[0] * 2,
            chessboardSize[1] * markerLength + (chessboardSize[1] - 1) * markerSeparation + pageBorder[1] * 2) as surface:
            context = cairo.Context(surface)

            context.set_source_rgba(0.5, 0.5, 0.5, 1.0)
            context.rectangle(0, 0,
                chessboardSize[0] * markerLength + (chessboardSize[0] - 1) * markerSeparation + pageBorder[0] * 2,
                chessboardSize[1] * markerLength + (chessboardSize[1] - 1) * markerSeparation + pageBorder[1] * 2)
            context.fill()

            context.set_source_rgba(1.0, 1.0, 1.0, 1.0)
            context.rectangle(pageBorder[0], pageBorder[1],
                chessboardSize[0] * markerLength + (chessboardSize[0] - 1) * markerSeparation,
                chessboardSize[1] * markerLength + (chessboardSize[1] - 1) * markerSeparation)
            context.fill()

            for bx in range(chessboardSize[0]):
                for by in range(chessboardSize[1]):
                    MarkerPrinter.__DrawBlock(
                        context = context,
                        dictionary = dictionary,
                        markerLength = markerLength,
                        borderBits = borderBits,
                        chessboardSize = chessboardSize,
                        squareLength = markerLength + markerSeparation,
                        firstMarkerID = firstMarker,
                        blockX = bx,
                        blockY = by,
                        pageBorderX = pageBorder[0],
                        pageBorderY = pageBorder[1],
                        mode = "ARUCOGRID")

        if(subSize is not None):
            subDivide = (\
                chessboardSize[0] // subSize[0] + int(chessboardSize[0] % subSize[0] > 0),
                chessboardSize[1] // subSize[1] + int(chessboardSize[1] % subSize[1] > 0))

            subChessboardBlockX = np.clip ( np.arange(0, subSize[0] * subDivide[0] + 1, subSize[0]), 0, chessboardSize[0])
            subChessboardBlockY = np.clip ( np.arange(0, subSize[1] * subDivide[1] + 1, subSize[1]), 0, chessboardSize[1])

            subChessboardSliceX = subChessboardBlockX.astype(np.float) * (markerLength + markerSeparation)
            subChessboardSliceY = subChessboardBlockY.astype(np.float) * (markerLength + markerSeparation)

            subChessboardSliceX[-1] -= markerSeparation
            subChessboardSliceY[-1] -= markerSeparation

            for subXID in range(subDivide[0]):
                for subYID in range(subDivide[1]):
                    subName = name + \
                        "_X" + str(subChessboardBlockX[subXID]) + "_" + str(subChessboardBlockX[subXID+1]) + \
                        "_Y" + str(subChessboardBlockY[subYID]) + "_" + str(subChessboardBlockY[subYID+1])

                    with MarkerPrinter.surface[ext.upper()](
                        os.path.join(path, subName + ext),
                        subChessboardSliceX[subXID+1] - subChessboardSliceX[subXID] + pageBorder[0] * 2,
                        subChessboardSliceY[subYID+1] - subChessboardSliceY[subYID] + pageBorder[1] * 2) as surface:
                        context = cairo.Context(surface)

                        context.set_source_rgba(0.5, 0.5, 0.5, 1.0)
                        context.rectangle(0, 0,
                            subChessboardSliceX[subXID+1] - subChessboardSliceX[subXID] + pageBorder[0] * 2,
                            subChessboardSliceY[subYID+1] - subChessboardSliceY[subYID] + pageBorder[1] * 2)
                        context.fill()

                        context.set_source_rgba(1.0, 1.0, 1.0, 1.0)
                        context.rectangle(pageBorder[0], pageBorder[1],
                            subChessboardSliceX[subXID+1] - subChessboardSliceX[subXID],
                            subChessboardSliceY[subYID+1] - subChessboardSliceY[subYID])
                        context.fill()

                        for bx in range(subChessboardBlockX[subXID+1] - subChessboardBlockX[subXID]):
                            for by in range(subChessboardBlockY[subYID+1] - subChessboardBlockY[subYID]):
                                MarkerPrinter.__DrawBlock(
                                    context = context,
                                    dictionary = dictionary,
                                    markerLength = markerLength,
                                    borderBits = borderBits,
                                    chessboardSize = chessboardSize,
                                    squareLength = markerLength + markerSeparation,
                                    firstMarkerID = firstMarker,
                                    blockX = subChessboardBlockX[subXID] + bx,
                                    blockY = subChessboardBlockY[subYID] + by,
                                    originX = subChessboardBlockX[subXID],
                                    originY = subChessboardBlockY[subYID],
                                    pageBorderX = pageBorder[0],
                                    pageBorderY = pageBorder[1],
                                    mode = "ARUCOGRID")

if __name__ == '__main__':
    parser = ArgumentParser()

    # Save marker image parameters
    chessGroup = parser.add_argument_group('chess', 'Chessboard')
    arucoGroup = parser.add_argument_group('aruco', 'ArUco')
    arucoGridGroup = parser.add_argument_group('aruco_grid', 'ArUco grid')
    charucoGroup = parser.add_argument_group('charuco', 'ChArUco')
    exclusiveGroup = parser.add_mutually_exclusive_group()

    exclusiveGroup.add_argument(
        "--chess", action='store_true', default=False,
        help="Choose to save chessboard marker")

    exclusiveGroup.add_argument(
        "--aruco", action='store_true', default=False,
        help="Choose to save ArUco marker")

    exclusiveGroup.add_argument(
        "--aruco_grid", action='store_true', default=False,
        help="Choose to save ArUco grid marker")

    exclusiveGroup.add_argument(
        "--charuco", action='store_true', default=False,
        help="Choose to save ChArUco marker")

    # Utility functions parameters
    exclusiveGroup.add_argument(
        "--generate", dest="arucoDataFileName",
        help="Generate aruco data to FILE", metavar="FILE")

    exclusiveGroup.add_argument(
        "--list_dictionary", action='store_true', default=False,
        help="List predefined aruco dictionary")

    # Parameters
    # fileName
    parser.add_argument(
        "--file", dest="fileName", default="./image.pdf",
        help="Save marker image to FILE", metavar="FILE")
    for group in [chessGroup, arucoGroup, arucoGridGroup, charucoGroup]:
        group.add_argument(
            "--" + group.title + "_file", dest="fileName",
            help="Save marker image to FILE", metavar="FILE")

    # dictionary
    parser.add_argument(
        "--dictionary", dest="dictionary", default="DICT_ARUCO_ORIGINAL",
        help="Generate marker via predefined DICTIONARY aruco dictionary", metavar="DICTIONARY")
    for group in [arucoGroup, arucoGridGroup, charucoGroup]:
        group.add_argument(
            "--" + group.title + "_dictionary", dest="dictionary",
            help="Generate marker via predefined DICTIONARY aruco dictionary", metavar="DICTIONARY")

    # size
    parser.add_argument(
        "--size_x", dest="sizeX", default="16",
        help="Save marker image with N board width", metavar="N")
    parser.add_argument(
        "--size_y", dest="sizeY", default="9",
        help="Save marker image with N board height", metavar="N")

    for group in [chessGroup, arucoGridGroup, charucoGroup]:
        group.add_argument(
            "--" + group.title + "_size_x", dest="sizeX",
            help="Save marker image with N board width", metavar="N")
        group.add_argument(
            "--" + group.title + "_size_y", dest="sizeY",
            help="Save marker image with N board height", metavar="N")

    # length
    parser.add_argument(
        "--square_length", dest="squareLength", default="0.09",
        help="Save marker image with L square length (Unit: meter)", metavar="L")
    parser.add_argument(
        "--marker_length", dest="markerLength", default="0.07",
        help="Save marker image with L marker length (Unit: meter)", metavar="L")
    parser.add_argument(
        "--marker_separation", dest="markerSeparation", default="0.02",
        help="Save marker image with L separation length (Unit: meter)", metavar="L")

    for group in [chessGroup, charucoGroup]:
        group.add_argument(
            "--" + group.title + "_square_length", dest="squareLength",
            help="Save marker image with L blocks length (Unit: meter)", metavar="L")

    for group in [arucoGroup, arucoGridGroup, charucoGroup]:
        group.add_argument(
            "--" + group.title + "_marker_length", dest="markerLength",
            help="Save marker image with L marker length (Unit: meter)", metavar="L")

    for group in [arucoGridGroup]:
        group.add_argument(
            "--" + group.title + "_marker_separation", dest="markerSeparation",
            help="Save marker image with L gap length (Unit: meter)", metavar="L")

    # else
    parser.add_argument(
        "--marker_id", dest="markerID", default="0",
        help="Save marker image with ID marker", metavar="ID")
    parser.add_argument(
        "--first_marker", dest="firstMarker", default="0",
        help="Save marker image that start with ID marker", metavar="ID")
    parser.add_argument(
        "--border_bits", dest="borderBits", default="1",
        help="Save marker image with N border size", metavar="N")

    for group in [arucoGroup]:
        group.add_argument(
            "--" + group.title + "_marker_id", dest="markerID",
            help="Save marker image with ID marker", metavar="ID")

    for group in [arucoGridGroup]:
        group.add_argument(
            "--" + group.title + "_first_marker", dest="firstMarker",
            help="Save marker image that start with ID marker", metavar="ID")

    for group in [arucoGroup, arucoGridGroup, charucoGroup]:
        group.add_argument(
            "--" + group.title + "_border_bits", dest="borderBits",
            help="Save marker image with N border size", metavar="N")

    # sub size
    parser.add_argument(
        "--sub_size_x", dest="subSizeX", default="0",
        help="Save marker image with N chuck width", metavar="N")
    parser.add_argument(
        "--sub_size_y", dest="subSizeY", default="0",
        help="Save marker image with N chuck height", metavar="N")

    for group in [chessGroup, arucoGridGroup, charucoGroup]:
        group.add_argument(
            "--" + group.title + "_sub_size_x", dest="subSizeX",
            help="Save marker image with N chuck width", metavar="N")
        group.add_argument(
            "--" + group.title + "_sub_size_y", dest="subSizeY",
            help="Save marker image with N chuck height", metavar="N")

    # page border
    parser.add_argument(
        "--page_border_x", dest="pageBorderX", default="0",
        help="Save with page border width L length (Unit: meter)", metavar="L")
    parser.add_argument(
        "--page_border_y", dest="pageBorderY", default="0",
        help="Save with page border height L length (Unit: meter)", metavar="L")

    for group in [chessGroup, arucoGroup, arucoGridGroup, charucoGroup]:
        group.add_argument(
            "--" + group.title + "_page_border_x", dest="pageBorderX", default="0",
            help="Save with page border width L length (Unit: meter)", metavar="L")
        group.add_argument(
            "--" + group.title + "_page_border_y", dest="pageBorderY", default="0",
            help="Save with page border height L length (Unit: meter)", metavar="L")

    # Run
    args = parser.parse_args()

    if(args.arucoDataFileName is not None):
        print("Generate aruco data to: " + args.arucoDataFileName)
        SaveArucoDictBytesList(args.arucoDataFileName)

    elif(args.list_dictionary):
        print("List predefined aruco dictionary")
        for i in MarkerPrinter.arucoDictBytesList.keys():
            print(i)

    elif(args.chess):
        try:
            sizeX = int(args.sizeX)
            sizeY = int(args.sizeY)
            squareLength = float(args.squareLength)
            subSizeX = int(args.subSizeX)
            subSizeY = int(args.subSizeY)
            pageBorderX = float(args.pageBorderX)
            pageBorderY = float(args.pageBorderY)
        except ValueError as e:
            warnings.warn(str(e))
        else:
            print("Save chessboard marker with parms: " + \
                    str({ \
                        "fileName": args.fileName, \
                        "sizeX": sizeX, \
                        "sizeY": sizeY, \
                        "squareLength": squareLength, \
                        "subSizeX": subSizeX, \
                        "subSizeY": subSizeY, \
                        "pageBorderX": pageBorderX, \
                        "pageBorderY": pageBorderY, \
                    }))

            subSize = None

            if(subSizeX > 0):
                if(subSizeY > 0):
                    subSize = (subSizeX, subSizeY)
                else:
                    subSize = (subSizeX, sizeY)
            else:
                if(subSizeY > 0):
                    subSize = (sizeX, subSizeY)
                else:
                    subSize = None

            # Gen
            MarkerPrinter.GenChessMarkerImage(args.fileName, (sizeX, sizeY), squareLength, subSize = subSize, pageBorder = (pageBorderX, pageBorderY))

    elif(args.aruco):
        try:
            markerLength = float(args.markerLength)
            markerID = int(args.markerID)
            borderBits = int(args.borderBits)
            pageBorderX = float(args.pageBorderX)
            pageBorderY = float(args.pageBorderY)
        except ValueError as e:
            warnings.warn(str(e))
        else:
            print("Save ArUco marker with parms: " + \
                    str({ \
                        "fileName": args.fileName, \
                        "dictionary": args.dictionary, \
                        "markerLength": markerLength, \
                        "markerID": markerID, \
                        "borderBits": borderBits, \
                        "pageBorderX": pageBorderX, \
                        "pageBorderY": pageBorderY, \
                    }))

            # Gen
            MarkerPrinter.GenArucoMarkerImage(args.fileName, args.dictionary, markerID, markerLength, borderBits=borderBits, pageBorder = (pageBorderX, pageBorderY))

    elif(args.aruco_grid):
        try:
            sizeX = int(args.sizeX)
            sizeY = int(args.sizeY)
            markerLength = float(args.markerLength)
            markerSeparation = float(args.markerSeparation)
            firstMarker = int(args.firstMarker)
            borderBits = int(args.borderBits)
            subSizeX = int(args.subSizeX)
            subSizeY = int(args.subSizeY)
            pageBorderX = float(args.pageBorderX)
            pageBorderY = float(args.pageBorderY)
        except ValueError as e:
            warnings.warn(str(e))
        else:
            print("Save ArUco grid marker with parms: " + \
                    str({ \
                        "fileName": args.fileName, \
                        "dictionary": args.dictionary, \
                        "sizeX": sizeX, \
                        "sizeY": sizeY, \
                        "markerLength": markerLength, \
                        "markerSeparation": markerSeparation, \
                        "firstMarker": firstMarker, \
                        "borderBits": borderBits, \
                        "subSizeX": subSizeX, \
                        "subSizeY": subSizeY, \
                        "pageBorderX": pageBorderX, \
                        "pageBorderY": pageBorderY, \
                    }))

            subSize = None

            if(subSizeX > 0):
                if(subSizeY > 0):
                    subSize = (subSizeX, subSizeY)
                else:
                    subSize = (subSizeX, sizeY)
            else:
                if(subSizeY > 0):
                    subSize = (sizeX, subSizeY)
                else:
                    subSize = None

            # Gen
            MarkerPrinter.GenArucoGridMarkerImage(args.fileName, args.dictionary, (sizeX, sizeY), markerLength, markerSeparation, firstMarker, borderBits=borderBits, subSize=subSize, pageBorder = (pageBorderX, pageBorderY))

    elif(args.charuco):
        try:
            sizeX = int(args.sizeX)
            sizeY = int(args.sizeY)
            squareLength = float(args.squareLength)
            markerLength = float(args.markerLength)
            borderBits = int(args.borderBits)
            subSizeX = int(args.subSizeX)
            subSizeY = int(args.subSizeY)
            pageBorderX = float(args.pageBorderX)
            pageBorderY = float(args.pageBorderY)
        except ValueError as e:
            warnings.warn(str(e))
        else:
            print("Save ChArUco marker with parms: " + \
                    str({ \
                        "fileName": args.fileName, \
                        "dictionary": args.dictionary, \
                        "sizeX": sizeX, \
                        "sizeY": sizeY, \
                        "squareLength": squareLength, \
                        "markerLength": markerLength, \
                        "borderBits": borderBits, \
                        "subSizeX": subSizeX, \
                        "subSizeY": subSizeY, \
                        "pageBorderX": pageBorderX, \
                        "pageBorderY": pageBorderY, \
                    }))

            subSize = None

            if(subSizeX > 0):
                if(subSizeY > 0):
                    subSize = (subSizeX, subSizeY)
                else:
                    subSize = (subSizeX, sizeY)
            else:
                if(subSizeY > 0):
                    subSize = (sizeX, subSizeY)
                else:
                    subSize = None

            # Gen
            MarkerPrinter.GenCharucoMarkerImage(args.fileName, args.dictionary, (sizeX, sizeY), squareLength, markerLength, borderBits=borderBits, subSize=subSize, pageBorder = (pageBorderX, pageBorderY))

    else:
        parser.print_help()
